/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.biz.validate;

import com.abssqr.plat.biz.validate.validator.BigDecimalValidator;
import com.abssqr.plat.biz.validate.validator.CollectionValidator;
import com.abssqr.plat.biz.validate.validator.DateValidator;
import com.abssqr.plat.biz.validate.validator.DtValidator;
import com.abssqr.plat.biz.validate.validator.EnumsValidator;
import com.abssqr.plat.biz.validate.validator.MapValidator;
import com.abssqr.plat.biz.validate.validator.MoneyValidator;
import com.abssqr.plat.biz.validate.validator.NotNullValidator;
import com.abssqr.plat.biz.validate.validator.RangeValidator;
import com.abssqr.plat.biz.validate.validator.StringValidator;
import com.abssqr.plat.biz.validate.validator.ValueSetValidator;
import com.general.system.common.util.LogUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 校验器工具类
 * @author guozhenhua
 * @version com.loan.biz.service.util.validation: ValidateUtil.java, v 0.1 2018-12-16 11:36 PM guozhenhua Exp $
 */
public class ValidateUtil {
    
    private final static Logger LOGGER = LoggerFactory.getLogger(
            ValidateUtil.class);
    
    private static ValidateServiceImpl validateService = new ValidateServiceImpl();
    
    static {
        validateService.register(new CollectionValidator());
        validateService.register(new DateValidator());
        validateService.register(new DtValidator());
        validateService.register(new EnumsValidator());
        validateService.register(new MapValidator());
        validateService.register(new NotNullValidator());
        validateService.register(new RangeValidator());
        validateService.register(new StringValidator());
        validateService.register(new BigDecimalValidator());
        validateService.register(new ValueSetValidator());
        validateService.register(new MoneyValidator());
        LogUtil.info(LOGGER, "校验器工具类初始化完毕ValidateUtil");
    }
    
    /**
     * 校验请求对象参数,校验结果ValidatorError返回值交给调用者,由调用者来处理校验结果
     * @param targetObj
     * @return
     */
    public static ValidatorError validate(Object targetObj) {
        return validateService.validate(targetObj);
    }
    
    /**
     * 校验结果的描述信息提取
     * @param targetObj
     * @param validatorError
     * @return
     */
    public static String collectErrorsLog(Object targetObj, ValidatorError validatorError) {
        return validateService.collectErrorsLog(targetObj, validatorError);
    }
    
    /**
     * 校验请求对象参数,参数校验不通过直接输出异常
     * @param targetObj
     */
    public static void validateNoRes(Object targetObj) {
        ValidatorError validatorError = validateService.validate(targetObj);
        
        if (validatorError.hasErrors()) {
            String errorMsg = validateService.collectErrorsLog(targetObj, validatorError);
            LogUtil.error(LOGGER, "校验器校验失败,校验对象:{0},结果:{1}", targetObj, errorMsg);
            throw new IllegalArgumentException(errorMsg);
        }
    }
    
}
