/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.biz.validate.validator;

import com.abssqr.plat.biz.validate.AFieldValidator;
import com.abssqr.plat.biz.validate.FieldError;
import com.abssqr.plat.biz.validate.ValidateService;
import com.abssqr.plat.common.facade.validation.VldMoney;
import com.general.system.common.model.Money;
import com.general.system.common.util.MoneyUtil;

/**
 * 注解@{@link VldMoney}的校验类实现
 * 
 * @author zhenxuan.luo
 * @version $Id: MoneyValidator.java, v 0.1 2015年5月19日 上午11:01:04 zhenxuan.luo Exp $
 */
public final class MoneyValidator extends AFieldValidator<VldMoney> {
    /**
     * 默认构造器
     */
    public MoneyValidator() {
        super(VldMoney.class);
    }

    @Override
    public boolean isValid(ValidateService validateService, VldMoney annotationObj,
                           Object validTargetFieldObj, FieldError errorConext) {
        boolean isValid = false;
        try {
            return isValid = validate(annotationObj, validTargetFieldObj);
        } finally {
            if (!isValid) {
                errorConext.setErrorMsg(annotationObj.message());
            }
        }
    }

    /**
     * 校验
     * @param annotationObj 校验注解对象
     * @param validTargetFieldObj 需要校验的字段对象
     * @return true:校验通过,反之返回false
     */
    private boolean validate(VldMoney annotationObj, Object validTargetFieldObj) {
        if (validTargetFieldObj == null) {
            return (annotationObj.nullable()) ? true : false;
        }
        if (validTargetFieldObj instanceof Money) {
            Money money = (Money) validTargetFieldObj;
            if (annotationObj.zeroable() && MoneyUtil.compare(money, new Money()) >= 0) {
                return true;
            } else if (!annotationObj.zeroable() && MoneyUtil.compare(money, new Money()) > 0) {
                return true;
            }
        }
        return false;
    }
}
