/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.biz.validate.validator;

import com.abssqr.plat.biz.validate.AFieldValidator;
import com.abssqr.plat.biz.validate.FieldError;
import com.abssqr.plat.biz.validate.ValidateService;
import com.abssqr.plat.common.facade.validation.VldValueSet;
import org.apache.commons.lang3.StringUtils;

/**
 * 注解@{@link VldValueSet}的校验类实现
 * @author zhenxuan.luo
 * @version $Id: RangeValidator.java, v 0.1 2015年5月19日 上午10:56:18 zhenxuan.luo Exp $
 */
public final class ValueSetValidator extends AFieldValidator<VldValueSet> {
    /**
     * 默认构造器
     */
    public ValueSetValidator() {
        super(VldValueSet.class);
    }
    
    @Override
    public boolean isValid(ValidateService validateService, VldValueSet annotationObj,
                           Object validTargetFieldObj, FieldError errorConext) {
        boolean isValid = false;
        try {
            return isValid = validate(annotationObj, validTargetFieldObj);
        } finally {
            if (!isValid) {
                errorConext.setErrorMsg(annotationObj.message());
            }
        }
    }
    
    /**
     * 校验
     * @param annotationObj 校验注解对象
     * @param validTargetFieldObj 需要校验的字段对象
     * @return true:校验通过,反之返回false
     */
    private boolean validate(VldValueSet annotationObj, Object validTargetFieldObj) {
        if (validTargetFieldObj == null) {
            return (annotationObj.nullable()) ? true : false;
        }
        
        if (annotationObj.valueSet() == null || annotationObj.valueSet().length == 0) {
            return true;
        }
        
        String targetValue = (String) validTargetFieldObj;
        for (String value : annotationObj.valueSet()) {
            if (StringUtils.equals(StringUtils.trim(value), targetValue)) {
                return true;
            }
        }
        
        return false;
    }
}
