/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.commonTask;

import java.util.Date;
import java.util.Map;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherExecutorTemp;
import com.abssqr.plat.biz.shared.handler.CommonTaskHandler;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.SystemDateUtil;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * CommonTask
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.task.task: CommonTaskExecutor.java, v 0.1 2018-01-08 02:15 hongwei.wang Exp $
 */
@Component
public class CommonTaskExecutor extends DispatcherExecutorTemp<CommonTask> {
    private final static Logger logger = LoggerFactory.getLogger(
        CommonTaskExecutor.class);
    @Autowired
    CommonTaskRepository commonTaskRepository;

    private Map<CommonTaskTypeEnum, CommonTaskHandler> commonTaskHanlderMap = Maps.newHashMap();

    public void registerHandler(CommonTaskTypeEnum commonTaskTypeEnum, CommonTaskHandler commonTaskHandler) {
        commonTaskHanlderMap.put(commonTaskTypeEnum, commonTaskHandler);
    }

    @Override
    protected CommonTask doExecute(CommonTask node) {
        CommonTaskHandler commonTaskHandler = commonTaskHanlderMap.get(node.getTaskType());
        if (commonTaskHandler == null) {
            return markFail(node, "找不到对应的处理器");
        }
        return commonTaskHandler.execute(node);
    }

    @Override
    protected CommonTask markSuccess(CommonTask node) {
        commonTaskRepository.updateTaskStatus(node.getId(), TaskStatusEnum.SUCCESS, node.getTaskStatus(), "succ");
        return node;
    }

    @Override
    protected CommonTask markFail(CommonTask node, String failMessage) {
        // 因异常引起的重试
        node.setTaskStatus(TaskStatusEnum.FAILURE);
        commonTaskRepository.updateTaskExep(new Date(), node.getId(), failMessage);
        return node;
    }

    @Override
    protected CommonTask markException(CommonTask node, String failMessage) {
        Date nextTime = calcNextTime(node.getExecTimes() == null ? 1 : node.getExecTimes().intValue());

        DateTimeUtil.addSecond(SystemDateUtil.getSystemDate(), 30);
        node.setTaskStatus(TaskStatusEnum.INIT);
        commonTaskRepository.updateTaskExep(nextTime, node.getId(), failMessage);
        return node;
    }

    @Override
    protected String getName() {
        return "CommonTaskExecutor";
    }

    /**
     * 根据执行次数计算下次执行时间
     *
     * @param execTimes
     * @return
     */
    private Date calcNextTime(int execTimes) {

        if (execTimes < 10) {
            return DateTimeUtil.addSecond(SystemDateUtil.getSystemDate(), 30 * execTimes);
        } else {
            return DateTimeUtil.addSecond(SystemDateUtil.getSystemDate(), 60 * 30);
        }

    }
}
