/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.commonTask;

import java.util.List;
import java.util.stream.Collectors;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.core.service.host.HostManager;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * 资产转让
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.task.task: PlanAstStatTaskLoader.java, v 0.1 2018-01-08 00:59 hongwei.wang Exp $
 */
@Component
public class CommonTaskLoader implements DispatcherLoader<CommonTask> {
    @Autowired
    CommonTaskRepository commonTaskRepository;
    @Autowired
    TransactionTemplate  transactionTemplate;
    @Autowired
    HostManager          hostManager;
    @Autowired
    JobControlRepository jobControlRepository;

    /**
     * 装载待分发节点
     *
     * @param size
     * @return
     */
    @Override
    public List<CommonTask> loadToDoNodes(int size) {

        return transactionTemplate.execute(transactionStatus -> {

            // 太多乐观锁冲突报错了，还是加个锁吧
            jobControlRepository.lockByJobName(CommonTaskStarter.JOB_NAME);

            List<CommonTask> tasks = commonTaskRepository.selectInitTask(
                Lists.newArrayList(CommonTaskTypeEnum.AST_MOD_CHECK), SystemDateUtil.getSystemDate(), size);
            if (CollectionUtils.isEmpty(tasks)) {
                return Lists.newArrayList();
            }

            List<Long> ids = tasks.stream().map(CommonTask::getId).collect(Collectors.toList());
            int count = commonTaskRepository.updateTaskExec(SystemDateUtil.getSystemDate(),
                hostManager.getHostInfo().getHostName(),
                tasks);
            VarChecker.checkEquals(count, tasks.size(),
                "[TaskLoader.loadToDoNodes]更新任务数跟期望数不一致，实际更新：{0}，期望更新：{1} - {2}",
                count,
                tasks.size(),
                ids);
            return tasks;
        });
    }
}
