package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstPackDO;
import java.util.List;
import java.util.Date;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.paging.AstPackPagePage;
import com.abssqr.plat.common.dal.mysql.auto.mapper.AstPackDOMapper;

/**
* The Table abs_ast_pack.
* 资产包
*/
@Repository
public class AstPackDAO{

    @Autowired
    private AstPackDOMapper astPackDOMapper;

    /**
     * desc:插入表:abs_ast_pack.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(AstPackDO entity){
        return astPackDOMapper.insert(entity);
    }
    /**
     * desc:批量插入表:abs_ast_pack.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<AstPackDO> list){
        astPackDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:根据主键删除数据:abs_ast_pack.<br/>
     * @param id id
     * @return int
     */
    
    public int deleteById(Long id){
        return astPackDOMapper.deleteById(id);
    }
    /**
     * desc:根据主键获取数据:abs_ast_pack.<br/>
     * @param id id
     * @return AstPackDO
     */
    
    public AstPackDO getById(Long id){
        return astPackDOMapper.getById(id);
    }
    /**
     * desc:根据唯一约束AbsPackPackNo更新表:abs_ast_pack.<br/>
     * @param entity entity
     * @return int
     */
    
    public int updateByAbsPackPackNo(AstPackDO entity){
        return astPackDOMapper.updateByAbsPackPackNo(entity);
    }
    /**
     * desc:根据唯一约束AbsPackPackNo更新表:abs_ast_pack.<br/>
     * @param packDate packDate
     * @param packSubmitDate packSubmitDate
     * @param packNo packNo
     * @return int
     */
    
    public int updateSealPack(Date packDate,Date packSubmitDate,String packNo){
        return astPackDOMapper.updateSealPack(packDate, packSubmitDate, packNo);
    }
    /**
     * desc:根据唯一约束AbsPackPackNo删除数据:abs_ast_pack.<br/>
     * @param packNo packNo
     * @return int
     */
    
    public int deleteByAbsPackPackNo(String packNo){
        return astPackDOMapper.deleteByAbsPackPackNo(packNo);
    }
    /**
     * desc:根据唯一约束AbsPackPackNo获取数据:abs_ast_pack.<br/>
     * @param packNo packNo
     * @return AstPackDO
     */
    
    public AstPackDO getByAbsPackPackNo(String packNo){
        return astPackDOMapper.getByAbsPackPackNo(packNo);
    }
    /**
     * desc:根据普通索引AbsPackPlanNo获取数据:abs_ast_pack.<br/>
     * @param planNo planNo
     * @return List<AstPackDO>
     */
    
    public List<AstPackDO> queryByAbsPackPlanNo(String planNo){
        return astPackDOMapper.queryByAbsPackPlanNo(planNo);
    }
    /**
     * desc:根据资产包名查询数量.<br/>
     * @param packName packName
     * @return Long
     */
    
    public Long cntByPackName(String packName){
        return astPackDOMapper.cntByPackName(packName);
    }
    /**
     * desc:分页获取资产包.<br/>
     * @param astPackPage astPackPage
     * @return AstPackPagePage
     */
    
    public AstPackPagePage getList(AstPackPagePage astPackPage){
        int total = astPackDOMapper.getListCount(astPackPage);
        if(total>0){
            astPackPage.setDatas(astPackDOMapper.getListResult(astPackPage));
        }else{
            astPackPage.setDatas(new ArrayList());
        }
        astPackPage.setTotal(total);
        return astPackPage;
    }
}
