package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.TaskGroupDO;
import java.util.List;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.HostTaskGroupCnt;
import com.abssqr.plat.common.dal.mysql.auto.mapper.TaskGroupDOMapper;

/**
* The Table abs_task_group.
* ABS_TASK_GROUP
*/
@Repository
public class TaskGroupDAO{

    @Autowired
    private TaskGroupDOMapper taskGroupDOMapper;

    /**
     * desc:插入表:abs_task_group.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(TaskGroupDO entity){
        return taskGroupDOMapper.insert(entity);
    }
    /**
     * desc:批量插入表:abs_task_group.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<TaskGroupDO> list){
        taskGroupDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:更新表:abs_task_group.<br/>
     * @param entity entity
     * @return int
     */
    
    public int update(TaskGroupDO entity){
        return taskGroupDOMapper.update(entity);
    }
    /**
     * desc:根据主键删除数据:abs_task_group.<br/>
     * @param id id
     * @return int
     */
    
    public int deleteById(Long id){
        return taskGroupDOMapper.deleteById(id);
    }
    /**
     * desc:根据主键获取数据:abs_task_group.<br/>
     * @param id id
     * @return TaskGroupDO
     */
    
    public TaskGroupDO getById(Long id){
        return taskGroupDOMapper.getById(id);
    }
    /**
     * desc:获取所有未完成的任务组.<br/>
     * @param jobName jobName
     * @return List<TaskGroupDO>
     */
    
    public List<TaskGroupDO> getAllUnFinishTaskGroupByJobName(String jobName){
        return taskGroupDOMapper.getAllUnFinishTaskGroupByJobName(jobName);
    }
    /**
     * desc:获取各个主机的正在运行的任务组.<br/>
     * @param jobName jobName
     * @return List<HostTaskGroupCnt>
     */
    
    public List<HostTaskGroupCnt> getHostTaskGroupCnt(String jobName){
        return taskGroupDOMapper.getHostTaskGroupCnt(jobName);
    }
    /**
     * desc:更改当前任务的状态.<br/>
     * @param id id
     * @param status status
     * @return int
     */
    
    public int updateTaskGroupStatusById(Long id,String status){
        return taskGroupDOMapper.updateTaskGroupStatusById(id, status);
    }
    /**
     * desc:根据HOSTIP和Disable状态 数据:abs_task_group.<br/>
     * @param jobName jobName
     * @param hostName hostName
     * @return List<TaskGroupDO>
     */
    
    public List<TaskGroupDO> getExecTaskGroupByHostIpAndJobName(String jobName,String hostName){
        return taskGroupDOMapper.getExecTaskGroupByHostIpAndJobName(jobName, hostName);
    }
    /**
     * desc:获取所有正在执行的任务区间.<br/>
     * @param jobName jobName
     * @return List<TaskGroupDO>
     */
    
    public List<TaskGroupDO> getAllExecTaskGroupByJobName(String jobName){
        return taskGroupDOMapper.getAllExecTaskGroupByJobName(jobName);
    }
    /**
     * desc:获取一个可分配的任务区间.<br/>
     * @param jobName jobName
     * @return TaskGroupDO
     */
    
    public TaskGroupDO getAvailableTaskGroupByJobName(String jobName){
        return taskGroupDOMapper.getAvailableTaskGroupByJobName(jobName);
    }
    /**
     * desc:获取所有已经分配任务区间的机器.<br/>
     * @param jobName jobName
     * @return List<TaskGroupDO>
     */
    
    public List<TaskGroupDO> getAllExeMachine(String jobName){
        return taskGroupDOMapper.getAllExeMachine(jobName);
    }
    /**
     * desc:更改当前任务为初始化状态.<br/>
     * @param id id
     * @return int
     */
    
    public int updateTaskGroupInit(Long id){
        return taskGroupDOMapper.updateTaskGroupInit(id);
    }
    /**
     * desc:更改当前任务为Disable=true的状态.<br/>
     * @param id id
     * @param disable disable
     * @param oldDisable oldDisable
     * @return int
     */
    
    public int updateTaskGroupDisable(Long id,Boolean disable,Boolean oldDisable){
        return taskGroupDOMapper.updateTaskGroupDisable(id, disable, oldDisable);
    }
    /**
     * desc:为机器分配任务.<br/>
     * @param id id
     * @param hostName hostName
     * @return int
     */
    
    public int updateTaskGroupOwner(Long id,String hostName){
        return taskGroupDOMapper.updateTaskGroupOwner(id, hostName);
    }
    /**
     * desc:为机器分配任务.<br/>
     * @param id id
     * @return int
     */
    
    public int updateTaskGroupFinish(Long id){
        return taskGroupDOMapper.updateTaskGroupFinish(id);
    }
}
