package com.abssqr.plat.common.dal.mysql.auto.mapper;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstPackDO;
import java.util.List;
import java.util.Date;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.paging.AstPackPagePage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstPackItem;
import org.apache.ibatis.annotations.Param;

/**
 * 由于需要对分页支持,请直接使用对应的DAO类
 * The Table abs_ast_pack.
 * 资产包
 */
public interface AstPackDOMapper{

    /**
     * desc:插入表:abs_ast_pack.<br/>
     * @param entity entity
     * @return int
     */
    int insert(AstPackDO entity);
    /**
     * desc:批量插入表:abs_ast_pack.<br/>
     * @param list list
     * @return int
     */
    int insertBatch(List<AstPackDO> list);
    /**
     * desc:根据主键删除数据:abs_ast_pack.<br/>
     * @param id id
     * @return int
     */
    int deleteById(Long id);
    /**
     * desc:根据主键获取数据:abs_ast_pack.<br/>
     * @param id id
     * @return AstPackDO
     */
    AstPackDO getById(Long id);
    /**
     * desc:根据唯一约束AbsPackPackNo更新表:abs_ast_pack.<br/>
     * @param entity entity
     * @return int
     */
    int updateByAbsPackPackNo(AstPackDO entity);
    /**
     * desc:根据唯一约束AbsPackPackNo更新表:abs_ast_pack.<br/>
     * @param packDate packDate
     * @param packSubmitDate packSubmitDate
     * @param packNo packNo
     * @return int
     */
    int updateSealPack(@Param("packDate")Date packDate,@Param("packSubmitDate")Date packSubmitDate,@Param("packNo")String packNo);
    /**
     * desc:根据唯一约束AbsPackPackNo删除数据:abs_ast_pack.<br/>
     * @param packNo packNo
     * @return int
     */
    int deleteByAbsPackPackNo(@Param("packNo")String packNo);
    /**
     * desc:根据唯一约束AbsPackPackNo获取数据:abs_ast_pack.<br/>
     * @param packNo packNo
     * @return AstPackDO
     */
    AstPackDO getByAbsPackPackNo(@Param("packNo")String packNo);
    /**
     * desc:根据普通索引AbsPackPlanNo获取数据:abs_ast_pack.<br/>
     * @param planNo planNo
     * @return List<AstPackDO>
     */
    List<AstPackDO> queryByAbsPackPlanNo(@Param("planNo")String planNo);
    /**
     * desc:根据资产包名查询数量.<br/>
     * @param packName packName
     * @return Long
     */
    Long cntByPackName(@Param("packName")String packName);
    /**
     * desc:分页获取资产包.<br/>
     * @param astPackPage astPackPage
     * @return int
     */
    int getListCount(AstPackPagePage astPackPage);
    /**
     * desc:分页获取资产包.<br/>
     * @param astPackPage astPackPage
     * @return List<AstPackItem>
     */
    List<AstPackItem> getListResult(AstPackPagePage astPackPage);
}
