package com.abssqr.plat.common.dal.mysql.auto.mapper;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.LoanDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.LoanListPage;
import java.util.List;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.LoanPropCntStat;
import com.abssqr.plat.common.dal.mysql.auto.paging.CapitalAstMatchPage;
import java.lang.String;
import com.abssqr.plat.common.dal.mysql.auto.paging.CapitalAstUnMatchPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstPoolLoanStat;
import com.abssqr.plat.common.dal.mysql.auto.paging.AstPoolLoanPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstPoolLoan;
import com.abssqr.plat.common.dal.mysql.auto.paging.PackAstPage;
import org.apache.ibatis.annotations.Param;

/**
 * 由于需要对分页支持,请直接使用对应的DAO类
 * The Table abs_loan.
 * 信贷资产
 */
public interface LoanDOMapper{

    /**
     * desc:插入表:abs_loan.<br/>
     * @param entity entity
     * @return int
     */
    int insert(LoanDO entity);
    /**
     * desc:根据唯一约束OutAstNo更新表:abs_loan.<br/>
     * @param entity entity
     * @return int
     */
    int updateByAstNo(LoanDO entity);
    /**
     * desc:根据唯一约束OutAstNo获取数据:abs_loan.<br/>
     * @param outAstNo outAstNo
     * @param managerOrgCode managerOrgCode
     * @return LoanDO
     */
    LoanDO getByOutAstNo(@Param("outAstNo")String outAstNo,@Param("managerOrgCode")String managerOrgCode);
    /**
     * desc:根据唯一约束OutAstNo获取数据:abs_loan.<br/>
     * @param outAstNo outAstNo
     * @param managerOrgCode managerOrgCode
     * @return LoanDO
     */
    LoanDO lockByOutAstNo(@Param("outAstNo")String outAstNo,@Param("managerOrgCode")String managerOrgCode);
    /**
     * desc:根据唯一约束AbsLoan获取数据:abs_loan.<br/>
     * @param astNo astNo
     * @return LoanDO
     */
    LoanDO getByAstNo(String astNo);
    /**
     * desc:根据唯一约束AbsLoan获取数据:abs_loan.<br/>
     * @param astNo astNo
     * @return LoanDO
     */
    LoanDO lockByAstNo(String astNo);
    /**
     * desc:getPageByOwnOrgCount.<br/>
     * @param loanList loanList
     * @return int
     */
    int getPageByOwnOrgCount(LoanListPage loanList);
    /**
     * desc:getPageByOwnOrgResult.<br/>
     * @param loanList loanList
     * @return List<LoanDO>
     */
    List<LoanDO> getPageByOwnOrgResult(LoanListPage loanList);
    /**
     * desc:统计机构当前资产规模.<br/>
     * @param lenderOrgCode lenderOrgCode
     * @return LoanPropCntStat
     */
    LoanPropCntStat statPlanBal(String lenderOrgCode);
    /**
     * desc:统计放款池当前持有资产规模(不包含已转出).<br/>
     * @param lenderOrgCode lenderOrgCode
     * @return LoanPropCntStat
     */
    LoanPropCntStat statSpvBal(String lenderOrgCode);
    /**
     * desc:资金资产匹配分页.<br/>
     * @param capitalAstMatch capitalAstMatch
     * @return int
     */
    int queryForCapitalAstMatchCount(CapitalAstMatchPage capitalAstMatch);
    /**
     * desc:资金资产匹配分页.<br/>
     * @param capitalAstMatch capitalAstMatch
     * @return List<String>
     */
    List<String> queryForCapitalAstMatchResult(CapitalAstMatchPage capitalAstMatch);
    /**
     * desc:资金资产解绑分页.<br/>
     * @param capitalAstUnMatch capitalAstUnMatch
     * @return int
     */
    int queryForCapitalAstUnMatchCount(CapitalAstUnMatchPage capitalAstUnMatch);
    /**
     * desc:资金资产解绑分页.<br/>
     * @param capitalAstUnMatch capitalAstUnMatch
     * @return List<String>
     */
    List<String> queryForCapitalAstUnMatchResult(CapitalAstUnMatchPage capitalAstUnMatch);
    /**
     * desc:根据资产池编号统计已打包为总共的本金余额和资产笔数.<br/>
     * @param astPoolNo astPoolNo
     * @return AstPoolLoanStat
     */
    AstPoolLoanStat statPrinAndCnt(String astPoolNo);
    /**
     * desc:根据资产池编号查询对应的资产.<br/>
     * @param astPoolLoan astPoolLoan
     * @return int
     */
    int getLoanListByAstPoolNoCount(AstPoolLoanPage astPoolLoan);
    /**
     * desc:根据资产池编号查询对应的资产.<br/>
     * @param astPoolLoan astPoolLoan
     * @return List<AstPoolLoan>
     */
    List<AstPoolLoan> getLoanListByAstPoolNoResult(AstPoolLoanPage astPoolLoan);
    /**
     * desc:根据资产包编号分页获取所有的资产.<br/>
     * @param packAst packAst
     * @return int
     */
    int getLoanPageByPackNoCount(PackAstPage packAst);
    /**
     * desc:根据资产包编号分页获取所有的资产.<br/>
     * @param packAst packAst
     * @return List<LoanDO>
     */
    List<LoanDO> getLoanPageByPackNoResult(PackAstPage packAst);
}
