package com.abssqr.plat.common.dal.mysql.auto.mapper;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanCapitalOrderDO;
import java.util.List;
import java.util.Date;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.paging.PlanCapitalOrderPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.CapitalOrderAmt;
import org.apache.ibatis.annotations.Param;

/**
 * 由于需要对分页支持,请直接使用对应的DAO类
 * The Table abs_plan_capital_order.
 * 计划资金交易订单
 */
public interface PlanCapitalOrderDOMapper{

    /**
     * desc:插入表:ABS_PLAN_CAPITAL_ORDER.<br/>
     * @param entity entity
     * @return int
     */
    int insert(PlanCapitalOrderDO entity);
    /**
     * desc:插入表:ABS_PLAN_CAPITAL_ORDER.<br/>
     * @param entity entity
     * @return int
     */
    int insertHis(PlanCapitalOrderDO entity);
    /**
     * desc:批量插入表:ABS_PLAN_CAPITAL_ORDER.<br/>
     * @param list list
     * @return int
     */
    int insertBatch(List<PlanCapitalOrderDO> list);
    /**
     * desc:根据唯一约束AbsPlanCapitalOrdr更新表:ABS_PLAN_CAPITAL_ORDER.<br/>
     * @param entity entity
     * @return int
     */
    int updateProcess(PlanCapitalOrderDO entity);
    /**
     * desc:失败的交易重新发起.<br/>
     * @param nextExecTime nextExecTime
     * @param seqNo seqNo
     * @return PlanCapitalOrderDO
     */
    PlanCapitalOrderDO restoreCapitalOrder(@Param("nextExecTime")Date nextExecTime,@Param("seqNo")String seqNo);
    /**
     * desc:根据唯一约束AbsPlanCapitalOrdr删除数据:ABS_PLAN_CAPITAL_ORDER.<br/>
     * @param seqNo seqNo
     * @param planNo planNo
     * @return int
     */
    int deleteBySeqno(@Param("seqNo")String seqNo,@Param("planNo")String planNo);
    /**
     * desc:根据唯一约束AbsPlanCapitalLog获取数据:abs_plan_capital_log.<br/>
     * @param seqNo seqNo
     * @return PlanCapitalOrderDO
     */
    PlanCapitalOrderDO lockBySeqno(@Param("seqNo")String seqNo);
    /**
     * desc:根据唯一约束AbsPlanCapitalLog获取数据:abs_plan_capital_log.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param transType transType
     * @return PlanCapitalOrderDO
     */
    PlanCapitalOrderDO lockActiveByPlanNo(@Param("accountDate")Date accountDate,@Param("planNo")String planNo,@Param("transType")String transType);
    /**
     * desc:根据唯一约束AbsPlanCapitalLog获取数据:abs_plan_capital_log.<br/>
     * @param planNo planNo
     * @return List<PlanCapitalOrderDO>
     */
    List<PlanCapitalOrderDO> getInitListByPlanNo(@Param("planNo")String planNo);
    /**
     * desc:查询存活支付单条数.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param transType transType
     * @param list list
     * @return Long
     */
    Long getActiveCntByCondition(@Param("accountDate")Date accountDate,@Param("planNo")String planNo,@Param("transType")String transType,@Param("list")List<String> list);
    /**
     * desc:根据状态分页查询订单.<br/>
     * @param planCapitalOrder planCapitalOrder
     * @return int
     */
    int getCapitalOrderByStatusCount(PlanCapitalOrderPage planCapitalOrder);
    /**
     * desc:根据状态分页查询订单.<br/>
     * @param planCapitalOrder planCapitalOrder
     * @return List<PlanCapitalOrderDO>
     */
    List<PlanCapitalOrderDO> getCapitalOrderByStatusResult(PlanCapitalOrderPage planCapitalOrder);
    /**
     * desc:查询执行次数较多的交易单.<br/>
     * @param accountDate accountDate
     * @param execTimes execTimes
     * @param list list
     * @return List<PlanCapitalOrderDO>
     */
    List<PlanCapitalOrderDO> queryFailCapitalOrder(@Param("accountDate")Date accountDate,@Param("execTimes")Long execTimes,@Param("list")List<String> list);
    /**
     * desc:查询失败数量.<br/>
     * @param execTimes execTimes
     * @return Long
     */
    Long getFailCount(Long execTimes);
    /**
     * desc:根据外部交易编码查询交易单用来做mock数据.<br/>
     * @param status status
     * @param noList noList
     * @return List<PlanCapitalOrderDO>
     */
    List<PlanCapitalOrderDO> queryOrderListByOutAcptNo(@Param("status")String status,@Param("noList")List<String> noList);
    /**
     * desc:根据计划编号查询支付金额和交易金额.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param transType transType
     * @return CapitalOrderAmt
     */
    CapitalOrderAmt getHisAmtByPlanNoAndActDateAndTransCode(@Param("accountDate")Date accountDate,@Param("planNo")String planNo,@Param("transType")String transType);
}
