/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.base;

import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;

import java.util.List;

/**
 * 债类的金额字段
 *
 * @author zhenxuan.luo
 * @version com.abssqr.common.model.base: DebtMoney.java, v 0.1 2017-12-05 下午17:20 zhenxuan.luo Exp $ 
 */
public class DebtMoney extends BaseEntity {
    
    /**
     * 本金
     */
    private Money prin      = new Money(0);
    /**
     * 本金
     */
    private Money ovdPrin   = new Money(0);
    /**
     * 利息
     */
    private Money intr      = new Money(0);
    /**
     * 利息
     */
    private Money ovdIntr   = new Money(0);
    /**
     * 本罚
     */
    private Money prinPenal = new Money(0);
    /**
     * 利罚
     */
    private Money intrPenal = new Money(0);
    /**
     * 费用
     */
    private Money fee       = new Money(0);
    /**
     * 费用
     */
    private Money ovdFee    = new Money(0);
    /**
     * 滞纳金
     */
    private Money feePenal  = new Money(0);
    
    /**
     * 其他费用，如服务费等
     */
    private Money otherFee = new Money(0);
    
    /**
     * 其他费用产生的滞纳金
     */
    private Money otherFeePenal = new Money(0);
    
    /**
     * 判断是否都为0
     *
     * @return
     */
    public boolean isZero() {
        this.check("");
        return getSum().isZero();
    }
    
    /**
     * 交易金额全为0
     */
    public void checkZero(String errorPrefix) {
        for (DebtMoneyFieldEnum fieldEnum : DebtMoneyFieldEnum.values()) {
            VarChecker.checkZero(this.getByFieldType(fieldEnum),
                    "[AmoutCheckFailed]{0}必须等于0 [{1}] -- {2}", fieldEnum.getDesc(),
                    this.getByFieldType(fieldEnum), errorPrefix);
        }
    }
    
    /**
     * 金额校验
     *
     * @param errorPrefix
     */
    public void check(String errorPrefix) {
        for (DebtMoneyFieldEnum fieldEnum : DebtMoneyFieldEnum.values()) {
            VarChecker.checkNonnegative(this.getByFieldType(fieldEnum),
                    "[AmoutCheckFailed]{0}必须大于等于0 [{1}] -- {2}", fieldEnum.getDesc(),
                    this.getByFieldType(fieldEnum), errorPrefix);
            
        }
    }
    
    /**
     * 获取所有金额之和
     *
     * @return
     */
    public Money getSum() {
        return this.prin.add(this.ovdPrin).add(this.intr).add(this.ovdIntr).add(this.prinPenal).add(this.intrPenal)
                       .add(this.fee).add(this.ovdFee).add(this.feePenal).add(this.otherFee).add(this.otherFeePenal);
    }
    
    /**
     * 获取本金
     *
     * @return
     */
    public Money getAllPrin() {
        return this.prin.add(this.ovdPrin);
    }
    
    /**
     * 所有利息
     * @return
     */
    public Money getAllIntr() {
        return this.intr.add(this.ovdIntr);
    }
    
    /**
     * 获取非本金非利息之外的罚息
     * @return
     */
    public Money getAllPenal() {
        return  this.intrPenal.add(this.feePenal).add(this.otherFeePenal);
    }
    
    /**
     * 取利息费和
     * @return 所有利息汇总(外部金额 ）
     */
    public Money getAllIntAndFee() {
        return this.intr.add(this.ovdIntr).add(this.prinPenal).add(this.intrPenal).add(this.fee).add(this.ovdFee)
                       .add(this.feePenal).add(this.otherFee).add(this.otherFeePenal);
    }
    
    public Money getAllIntOut() {
        return this.intr.add(this.ovdIntr).add(this.intrPenal).add(this.prinPenal);
    }
    
    /**
     * 取费用和
     * @return 所有费用汇总
     */
    public Money getAllIntAndFeeWithoutPenal() {
        return this.intr.add(this.ovdIntr).add(this.fee).add(this.ovdFee).add(this.otherFee);
    }
    
    /**
     * 金额校验
     *
     * @param errorPrefix
     */
    public void checkEqual(DebtMoney amt, List<DebtMoneyFieldEnum> ignoreChecks, String errorPrefix) {
        for (DebtMoneyFieldEnum debtMoneyFieldEnum : DebtMoneyFieldEnum.values()) {
            if (CollectionUtils.isNotEmpty(ignoreChecks) && ignoreChecks.contains(debtMoneyFieldEnum)) {
                continue;
            }
            VarChecker.checkEquals(this.getByFieldType(debtMoneyFieldEnum),
                    amt.getByFieldType(debtMoneyFieldEnum),
                    "[AmoutCheckFailed]{0}金额不相等 [left:{1}, right:{2}] -- {3}",
                    debtMoneyFieldEnum.getDesc(),
                    this.getByFieldType(debtMoneyFieldEnum), amt.getByFieldType(debtMoneyFieldEnum),
                    errorPrefix);
        }
    }
    
    /**
     * 金额校验
     *
     * @param errorPrefix
     */
    public void checkEqual(DebtMoney amt, String errorPrefix) {
        for (DebtMoneyFieldEnum debtMoneyFieldEnum : DebtMoneyFieldEnum.values()) {
            VarChecker.checkEquals(this.getByFieldType(debtMoneyFieldEnum),
                    amt.getByFieldType(debtMoneyFieldEnum),
                    "[AmoutCheckFailed]{0}金额不相等 [left:{1}, right:{2}] -- {3}",
                    debtMoneyFieldEnum.getDesc(),
                    this.getByFieldType(debtMoneyFieldEnum), amt.getByFieldType(debtMoneyFieldEnum),
                    errorPrefix);
        }
    }
    
    /**
     * 根据金额类型获取金额字段
     *
     * @param DebtMoneyFieldEnum
     * @return
     */
    public void addByFieldType(DebtMoneyFieldEnum DebtMoneyFieldEnum, Money money) {
        this.setByFieldType(DebtMoneyFieldEnum, this.getByFieldType(DebtMoneyFieldEnum).add(money));
    }
    
    public void subByFieldType(DebtMoneyFieldEnum DebtMoneyFieldEnum, Money money) {
        this.setByFieldType(DebtMoneyFieldEnum, this.getByFieldType(DebtMoneyFieldEnum).subtract(money));
    }
    
    /**
     * 根据金额类型获取金额字段
     *
     * @param DebtMoneyFieldEnum
     * @return
     */
    public void addByFieldType(DebtMoneyFieldEnum DebtMoneyFieldEnum, DebtMoney DebtMoney) {
        addByFieldType(DebtMoneyFieldEnum, DebtMoney.getByFieldType(DebtMoneyFieldEnum));
    }
    
    /**
     * 累计各项金额
     *
     * @param DebtMoney
     * @return
     */
    public DebtMoney addAll(DebtMoney DebtMoney, DebtMoneyFieldEnum... excluds) {
        List<DebtMoneyFieldEnum> excludsList = Lists.newArrayList();
        if (excluds != null) {
            excludsList = Lists.newArrayList(excluds);
        }
        
        for (DebtMoneyFieldEnum debtMoneyFieldEnum : DebtMoneyFieldEnum.values()) {
            if (excludsList.contains(debtMoneyFieldEnum)) {
                continue;
            }
            this.addByFieldType(debtMoneyFieldEnum, DebtMoney);
        }
        return this;
    }
    
    /**
     * 累计各项金额
     *
     * @param DebtMoney
     * @return
     */
    public DebtMoney subAll(DebtMoney DebtMoney) {
        for (DebtMoneyFieldEnum debtMoneyFieldEnum : DebtMoneyFieldEnum.values()) {
            this.subByFieldType(debtMoneyFieldEnum, DebtMoney.getByFieldType(debtMoneyFieldEnum));
        }
        return this;
    }
    
    /**
     * 根据金额类型复制
     *
     * @return
     */
    public DebtMoney cloneAll(DebtMoneyFieldEnum... excluds) {
        DebtMoney DebtMoney = new DebtMoney();
        List<DebtMoneyFieldEnum> excludsList = Lists.newArrayList();
        if (excluds != null) {
            excludsList = Lists.newArrayList(excluds);
        }
        
        for (DebtMoneyFieldEnum debtMoneyFieldEnum : DebtMoneyFieldEnum.values()) {
            if (excludsList.contains(debtMoneyFieldEnum)) {
                continue;
            }
            DebtMoney.setByFieldType(debtMoneyFieldEnum, this.getByFieldType(debtMoneyFieldEnum).clone());
        }
        return DebtMoney;
    }
    
    /**
     * 根据金额类型获取金额字段
     *
     * @param DebtMoneyFieldEnum
     * @return
     */
    public Money getByFieldType(DebtMoneyFieldEnum DebtMoneyFieldEnum) {
        switch (DebtMoneyFieldEnum) {
            case PRIN:
                return this.prin.clone();
            case OVD_PRIN:
                return this.ovdPrin.clone();
            case INTR:
                return this.intr.clone();
            case OVD_INTR:
                return this.ovdIntr.clone();
            case PRIN_ODUE_PNY:
                return this.prinPenal.clone();
            case INTR_ODUE_PNY:
                return this.intrPenal.clone();
            case FEE:
                return this.fee.clone();
            case OVD_FEE:
                return this.ovdFee.clone();
            case FEE_PNY:
                return this.feePenal.clone();
            case OTHER_FEE:
                return this.otherFee.clone();
            case OTHER_FEE_PNY:
                return this.otherFeePenal.clone();
            default:
                return null;
        }
    }
    
    /**
     * 根据金额类型获取金额字段
     *
     * @param DebtMoneyFieldEnum
     * @return
     */
    public Money getByFieldTypeWithoutClone(DebtMoneyFieldEnum DebtMoneyFieldEnum) {
        switch (DebtMoneyFieldEnum) {
            case PRIN:
                return this.prin;
            case OVD_PRIN:
                return this.ovdPrin;
            case INTR:
                return this.intr;
            case OVD_INTR:
                return this.ovdIntr;
            case PRIN_ODUE_PNY:
                return this.prinPenal;
            case INTR_ODUE_PNY:
                return this.intrPenal;
            case FEE:
                return this.fee;
            case OVD_FEE:
                return this.ovdFee;
            case FEE_PNY:
                return this.feePenal;
            case OTHER_FEE:
                return this.otherFee;
            case OTHER_FEE_PNY:
                return this.otherFeePenal;
            default:
                return null;
        }
    }
    
    /**
     * 根据金额类型获取金额字段
     *
     * @param DebtMoneyFieldEnum
     * @return
     */
    public void setByFieldType(DebtMoneyFieldEnum DebtMoneyFieldEnum, Money money) {
        switch (DebtMoneyFieldEnum) {
            case PRIN:
                this.prin = money.clone();
                break;
            case OVD_PRIN:
                this.ovdPrin = money.clone();
                break;
            case INTR:
                this.intr = money.clone();
                break;
            case OVD_INTR:
                this.ovdIntr = money.clone();
                break;
            case PRIN_ODUE_PNY:
                this.prinPenal = money.clone();
                break;
            case INTR_ODUE_PNY:
                this.intrPenal = money.clone();
                break;
            case FEE:
                this.fee = money.clone();
                break;
            case OVD_FEE:
                this.ovdFee = money.clone();
                break;
            case FEE_PNY:
                this.feePenal = money.clone();
                break;
            case OTHER_FEE:
                this.otherFee = money.clone();
                break;
            case OTHER_FEE_PNY:
                this.otherFeePenal = money.clone();
                break;
        }
    }
}