/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.base;

import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;

import java.util.List;

/**
 * 收益类的金额字段
 *
 * @author hanfei
 * @version com.abssqr.common.model.base: ProfitMoney.java, v 0.1 2020-02-19 下午17:20 hanfei Exp $
 */
public class ProfitMoney extends BaseEntity {
    
    /**
     * 利息
     */
    private Money intr      = new Money(0);
    /**
     * 利息
     */
    private Money ovdIntr   = new Money(0);
    /**
     * 本罚
     */
    private Money prinPenal = new Money(0);
    /**
     * 利罚
     */
    private Money intrPenal = new Money(0);
    /**
     * 费用
     */
    private Money fee       = new Money(0);
    /**
     * 费用
     */
    private Money ovdFee    = new Money(0);
    /**
     * 滞纳金
     */
    private Money feePenal  = new Money(0);
    
    /**
     * 其他费用，如服务费等
     */
    private Money otherFee = new Money(0);
    
    /**
     * 其他费用产生的滞纳金
     */
    private Money otherFeePenal = new Money(0);
    
    /**
     * 不包含本金
     */
    private List<DebtMoneyFieldEnum> excludsList = Lists.newArrayList(DebtMoneyFieldEnum.PRIN, DebtMoneyFieldEnum.OVD_PRIN);
    /**
     * 判断是否都为0
     *
     * @return
     */
    public boolean isZero() {
        this.check("");
        return getSum().isZero();
    }
    
    /**
     * 交易金额全为0
     */
    public void checkZero(String errorPrefix) {
        for (DebtMoneyFieldEnum fieldEnum : DebtMoneyFieldEnum.values()) {
            if (excludsList.contains(fieldEnum)) {
                continue;
            }
            VarChecker.checkZero(this.getByFieldType(fieldEnum),
                    "[AmoutCheckFailed]{0}必须等于0 [{1}] -- {2}", fieldEnum.getDesc(),
                    this.getByFieldType(fieldEnum), errorPrefix);
        }
    }
    
    /**
     * 金额校验
     *
     * @param errorPrefix
     */
    public void check(String errorPrefix) {
        for (DebtMoneyFieldEnum fieldEnum : DebtMoneyFieldEnum.values()) {
            if (excludsList.contains(fieldEnum)) {
                continue;
            }
            VarChecker.checkNonnegative(this.getByFieldType(fieldEnum),
                    "[AmoutCheckFailed]{0}必须大于等于0 [{1}] -- {2}", fieldEnum.getDesc(),
                    this.getByFieldType(fieldEnum), errorPrefix);
        }
    }
    
    /**
     * 转换模型
     * @return
     */
    public DebtMoney turn() {
        DebtMoney debtMoney = new DebtMoney();
        for (DebtMoneyFieldEnum debtMoneyFieldEnum : DebtMoneyFieldEnum.values()) {
            if (excludsList.contains(debtMoneyFieldEnum)) {
                continue;
            }
            debtMoney.addByFieldType(debtMoneyFieldEnum, this.getByFieldType(debtMoneyFieldEnum));
        }
        return debtMoney;
    }
    
    /**
     * 获取所有金额之和
     *
     * @return
     */
    public Money getSum() {
        return this.intr.add(this.ovdIntr).add(this.prinPenal).add(this.intrPenal)
                       .add(this.fee).add(this.ovdFee).add(this.feePenal).add(this.otherFee).add(this.otherFeePenal);
    }
    
    /**
     * 根据金额类型获取金额字段
     *
     * @param debtMoneyFieldEnum
     * @return
     */
    public void addByFieldType(DebtMoneyFieldEnum debtMoneyFieldEnum, Money money) {
        if (excludsList.contains(debtMoneyFieldEnum)) {
            return;
        }
        this.setByFieldType(debtMoneyFieldEnum, this.getByFieldType(debtMoneyFieldEnum).add(money));
    }
    /**
     * 根据金额类型获取金额字段
     *
     * @param DebtMoneyFieldEnum
     * @return
     */
    public void addByFieldType(DebtMoneyFieldEnum DebtMoneyFieldEnum, DebtMoney DebtMoney) {
        addByFieldType(DebtMoneyFieldEnum, DebtMoney.getByFieldType(DebtMoneyFieldEnum));
    }
    
    /**
     * 收益累加
     * @param profitMoney
     * @return
     */
    public ProfitMoney addAll(ProfitMoney profitMoney) {
        for (DebtMoneyFieldEnum debtMoneyFieldEnum : DebtMoneyFieldEnum.values()) {
            if (excludsList.contains(debtMoneyFieldEnum)) {
                continue;
            }
            this.addByFieldType(debtMoneyFieldEnum, profitMoney.getByFieldType(debtMoneyFieldEnum));
        }
        return this;
    }
    /**
     * 累计各项金额
     *
     * @param debtMoney
     * @return
     */
    public ProfitMoney addAllByDebt(DebtMoney debtMoney) {
        for (DebtMoneyFieldEnum debtMoneyFieldEnum : DebtMoneyFieldEnum.values()) {
            if (excludsList.contains(debtMoneyFieldEnum)) {
                continue;
            }
            this.addByFieldType(debtMoneyFieldEnum, debtMoney);
        }
        return this;
    }
    
    /**
     * 根据金额类型复制
     *
     * @return
     */
    public ProfitMoney cloneAllProfit() {
        ProfitMoney DebtMoney = new ProfitMoney();
        for (DebtMoneyFieldEnum debtMoneyFieldEnum : DebtMoneyFieldEnum.values()) {
            if (excludsList.contains(debtMoneyFieldEnum)) {
                continue;
            }
            DebtMoney.setByFieldType(debtMoneyFieldEnum, this.getByFieldType(debtMoneyFieldEnum).clone());
        }
        return DebtMoney;
    }
    
    /**
     * 根据金额类型获取金额字段
     *
     * @param DebtMoneyFieldEnum
     * @return
     */
    public Money getByFieldType(DebtMoneyFieldEnum DebtMoneyFieldEnum) {
        switch (DebtMoneyFieldEnum) {
            case INTR:
                return this.intr.clone();
            case OVD_INTR:
                return this.ovdIntr.clone();
            case PRIN_ODUE_PNY:
                return this.prinPenal.clone();
            case INTR_ODUE_PNY:
                return this.intrPenal.clone();
            case FEE:
                return this.fee.clone();
            case OVD_FEE:
                return this.ovdFee.clone();
            case FEE_PNY:
                return this.feePenal.clone();
            case OTHER_FEE:
                return this.otherFee.clone();
            case OTHER_FEE_PNY:
                return this.otherFeePenal.clone();
            default:
                return null;
        }
    }
   
    /**
     * 根据金额类型获取金额字段
     *
     * @param DebtMoneyFieldEnum
     * @return
     */
    public void setByFieldType(DebtMoneyFieldEnum DebtMoneyFieldEnum, Money money) {
        switch (DebtMoneyFieldEnum) {
            case INTR:
                this.intr = money.clone();
                break;
            case OVD_INTR:
                this.ovdIntr = money.clone();
                break;
            case PRIN_ODUE_PNY:
                this.prinPenal = money.clone();
                break;
            case INTR_ODUE_PNY:
                this.intrPenal = money.clone();
                break;
            case FEE:
                this.fee = money.clone();
                break;
            case OVD_FEE:
                this.ovdFee = money.clone();
                break;
            case FEE_PNY:
                this.feePenal = money.clone();
                break;
            case OTHER_FEE:
                this.otherFee = money.clone();
                break;
            case OTHER_FEE_PNY:
                this.otherFeePenal = money.clone();
                break;
        }
    }
    
    public Money getIntr() {
        return intr;
    }
    
    public void setIntr(Money intr) {
        this.intr = intr;
    }
    
    public Money getOvdIntr() {
        return ovdIntr;
    }
    
    public void setOvdIntr(Money ovdIntr) {
        this.ovdIntr = ovdIntr;
    }
    
    public Money getPrinPenal() {
        return prinPenal;
    }
    
    public void setPrinPenal(Money prinPenal) {
        this.prinPenal = prinPenal;
    }
    
    public Money getIntrPenal() {
        return intrPenal;
    }
    
    public void setIntrPenal(Money intrPenal) {
        this.intrPenal = intrPenal;
    }
    
    public Money getFee() {
        return fee;
    }
    
    public void setFee(Money fee) {
        this.fee = fee;
    }
    
    public Money getOvdFee() {
        return ovdFee;
    }
    
    public void setOvdFee(Money ovdFee) {
        this.ovdFee = ovdFee;
    }
    
    public Money getFeePenal() {
        return feePenal;
    }
    
    public void setFeePenal(Money feePenal) {
        this.feePenal = feePenal;
    }
    
    public Money getOtherFee() {
        return otherFee;
    }
    
    public void setOtherFee(Money otherFee) {
        this.otherFee = otherFee;
    }
    
    public Money getOtherFeePenal() {
        return otherFeePenal;
    }
    
    public void setOtherFeePenal(Money otherFeePenal) {
        this.otherFeePenal = otherFeePenal;
    }
    
    
}