/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.facade.model.report;

import com.general.system.common.model.BaseEntity;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.MessageUtil;
import org.apache.commons.lang3.StringUtils;

import java.util.Date;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.facade.model.report: DailyReportDateField.java, v 0.1 2019-06-28 20:06 zhenxuan.luo Exp $
 */
public class MonthReportDateField extends BaseEntity implements ReportDateField {
    
    private int year;
    
    private int month;
    
    @Override
    public void parse(String val) {
        
        if (StringUtils.isBlank(val) || val.length() != 6) {
            throw new IllegalArgumentException(MessageUtil.formatMsg("月报字段解析失败[{0}]", val));
        }
        year = Integer.parseInt(StringUtils.substring(val, 0, 4));
        month = Integer.parseInt(StringUtils.substring(val, 4, 6));
    }
    
    @Override
    public String desc() {
        StringBuilder builder = new StringBuilder();
        builder.append(year);
        if (month < 10) {
            builder.append(0);
        }
        builder.append(month);
        return builder.toString();
    }
    
    @Override
    public MonthReportDateField toNext() {
        month = month + 1;
        if (month > 12) {
            month = 1;
            year = year + 1;
        }
        return this;
    }
    
    @Override
    public Date getAccountDate() {
        
        String monthText = desc();
        // 当月第一天
        Date curMonth = DateTimeUtil.parseInTolerance(monthText);
        // 下个月第一天
        Date nextMonth = DateTimeUtil.addMonths(curMonth, 1);
        // 通过下个月的第一天，计算当月最后一天
        return DateTimeUtil.addDays(nextMonth, -1);
    }
    
    public Date getRptStartDate() {
        String monthText = desc();
        // 当月第一天
        Date curMonth = DateTimeUtil.parseInTolerance(monthText);
        
        return curMonth;
    }
    
    public boolean isEarlier(MonthReportDateField obj) {
        if (this.year > obj.year) {
            return false;
        } else if (this.year < obj.year) {
            return true;
        } else {
            if (this.month > obj.month) {
                return false;
            } else if (this.month < obj.month) {
                return true;
            } else {
                return false;
            }
        }
    }
}