/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.facade.utils;

import cn.hutool.core.util.NumberUtil;
import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.general.system.common.model.Money;

import java.math.BigDecimal;

/**
 * 金额/利率 >> 计算器
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.util: CalcUtil.java, v 0.1 2019-06-04 03:25 zhenxuan.luo Exp $ 
 */
public class CalcUtil {
    
    private static final BigDecimal VTAX_ADDED_RATE = new BigDecimal("0.12");
    
    private static final BigDecimal DEC_100 = AbssqrConstants.PERC_SCALER;
    
    /**
     * 计算金额分配
     * @param mon 待分配金额
     * @param fct1 分子
     * @param fct2 分母
     * @return
     */
    public static Money calcMoneyFactor(Money mon, BigDecimal fct1, BigDecimal fct2) {
        
        if (fct1 == null || fct1.compareTo(BigDecimal.ZERO) == 0) {
            return new Money();
        }
        if (fct2 == null || fct2.compareTo(BigDecimal.ZERO) == 0) {
            return new Money();
        }
        if (mon == null) {
            return new Money();
        }
        return new Money(calcMoneyFactorDec(mon, fct1, fct2));
    }
    
    public static Money calcMoneyRatio(Money mon, BigDecimal ratio) {
        return calcMoneyRatio(mon, ratio, AbssqrConstants.DECIMAL_ROUNDING_MODE);
    }
    
    public static Money calcMoneyRatio(Money mon, BigDecimal ratio, Integer roundingMode) {
        
        if (ratio == null || ratio.compareTo(BigDecimal.ZERO) == 0) {
            return new Money();
        }
        
        if (mon == null) {
            return new Money();
        }
        return new Money(mon.getAmount().multiply(ratio), roundingMode);
    }
    
    public static BigDecimal calcMoneyFactorDec(Money mon, BigDecimal fct1, BigDecimal fct2) {
        
        BigDecimal factor = fct1.divide(fct2, AbssqrConstants.DECIMAL_SCALE, AbssqrConstants.DECIMAL_ROUNDING_MODE);
        
        return mon.getAmount().multiply(factor);
    }
    
    public static BigDecimal divide(BigDecimal var1, BigDecimal var2) {
        if (var2.compareTo(BigDecimal.ZERO) == 0) {
            return BigDecimal.ZERO;
        } else {
            return var1.divide(var2, AbssqrConstants.DECIMAL_SCALE, AbssqrConstants.DECIMAL_ROUNDING_MODE);
        }
    }
    
    /**
     * 相除得出比例%值
     * @param var1
     * @param var2
     * @return
     */
    public static BigDecimal dividePercRate(BigDecimal var1, BigDecimal var2) {
        return dividePercRate(var1, var2, 2);
    }
    public static BigDecimal dividePercRate(BigDecimal var1, BigDecimal var2, int rund) {
        return NumberUtil.round(calcPercRate(divide(var1, var2)) , rund);
    }
    
    public static Money calcVatTaxForNet(Money net, BigDecimal taxRate) {
        BigDecimal taxAmt = net.getAmount().multiply(getVtaxAddedRate(taxRate));
        return new Money(taxAmt, AbssqrConstants.DECIMAL_ROUNDING_MODE);
    }

    public static Money calcVatConfirmForNet(Money net, BigDecimal taxRate) {
        BigDecimal vtaxAdded = getVtaxAddedRate(taxRate);
        BigDecimal confirmAmt = net.getAmount().divide(vtaxAdded);
        return new Money(confirmAmt, AbssqrConstants.DECIMAL_ROUNDING_MODE);
    }
    
    public static Money disbAmtWithTax(Money amt, BigDecimal taxRate) {
        BigDecimal vtaxAdded = getVtaxAddedRate(taxRate);
        
        BigDecimal vatRateCoef = BigDecimal.ONE.divide(BigDecimal.ONE.add(vtaxAdded), AbssqrConstants.DECIMAL_SCALE,
                AbssqrConstants.DECIMAL_ROUNDING_MODE);
        
        BigDecimal disbDecimal = amt.getAmount().multiply(vatRateCoef);
        
        return new Money(disbDecimal, AbssqrConstants.DECIMAL_ROUNDING_MODE);
    }
    
    public static BigDecimal getRealRate(BigDecimal rate) {
        return rate;
    }
    
    public static BigDecimal calcRealRate(BigDecimal rate) {
        if (null == rate) {
            return BigDecimal.ZERO;
        }
        return rate.divide(DEC_100, AbssqrConstants.DECIMAL_SCALE,
                AbssqrConstants.DECIMAL_ROUNDING_MODE);
    }
    
    public static BigDecimal calcPercRate(BigDecimal rate) {
        return rate.multiply(DEC_100);
    }
    
    private static BigDecimal getVtaxAddedRate(BigDecimal taxRate) {
        BigDecimal vTax = getRealRate(taxRate);
        //_tmp = (1+tax)/(1-(tax*0.12))
        BigDecimal vTaxAdded = (BigDecimal.ONE.add(vTax))
                                       .divide(BigDecimal.ONE.subtract(vTax.multiply(VTAX_ADDED_RATE)),
                                               AbssqrConstants.DECIMAL_SCALE,
                                               AbssqrConstants.DECIMAL_ROUNDING_MODE);
        return vTaxAdded.subtract(BigDecimal.ONE);
    }

}