/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanReportEntityDO;
import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.AstFlowTypeEnum;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.ReportAstStatTypeEnum;
import com.abssqr.plat.common.facade.enums.ReportDimTypeEnum;
import com.abssqr.plat.common.facade.model.report.ReportEntity;
import com.abssqr.plat.common.facade.model.report.ReportItem;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;

import java.util.List;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.convertor: PlanReportEntityConvert.java, v 0.1 2019-06-27 16:15 zhenxuan.luo Exp $ 
 */
public class PlanReportEntityConvert {
    
    public static List<PlanReportEntityDO> convert2DOs(List<ReportEntity> reportEntities) {
        List<PlanReportEntityDO> dos = Lists.newArrayList();
        
        if (CollectionUtils.isEmpty(reportEntities)) {
            return dos;
        }
        
        for (ReportEntity reportEntity : reportEntities) {
            dos.add(convert2DO(reportEntity));
        }
        
        return dos;
    }
    
    public static ReportEntity convert2Domain(PlanReportEntityDO var) {
        if (var == null) {
            return null;
        }
        
        ReportEntity instance = new ReportEntity();
        
        instance.setPlanNo(var.getPlanNo());
        instance.setEventCode(EventCodeEnum.getByCode(var.getEventCode()));
        ReportItem reportItem = new ReportItem();
        reportItem.setAstStatType(ReportAstStatTypeEnum.getByCode(var.getStatType()));
        reportItem.setAstStatVal(var.getStatVal());
        instance.setReportItem(reportItem);
        instance.setAstChg(var.getAstChg());
        DebtMoney amt = new DebtMoney();
        amt.setByFieldType(DebtMoneyFieldEnum.PRIN, var.getPrinAmt());
        amt.setByFieldType(DebtMoneyFieldEnum.INTR, var.getIntrAmt());
        amt.setByFieldType(DebtMoneyFieldEnum.FEE, var.getFeeAmt());
        amt.setByFieldType(DebtMoneyFieldEnum.OVD_PRIN, var.getOvdPrinAmt());
        amt.setByFieldType(DebtMoneyFieldEnum.OVD_INTR, var.getOvdIntrAmt());
        amt.setByFieldType(DebtMoneyFieldEnum.OVD_FEE, var.getOvdFeeAmt());
        amt.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, var.getPrinPenalAmt());
        amt.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, var.getIntrPenalAmt());
        amt.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, var.getFeePenalAmt());
        amt.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE, var.getOtherFeeAmt());
        amt.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE_PNY, var.getOtherFeePenalAmt());
        instance.setAmt(amt);
        instance.setUserId(var.getUserId());
        instance.setAstNo(var.getAstNo());
        instance.setReportDimType(ReportDimTypeEnum.getByCode(var.getRptDim()));
        instance.setAccountDate(var.getAccountDate());
        if (var.getCost()!=null){
            instance.setCost(var.getCost());
        }
        instance.setOrgDistPftAmt(var.getOrgDistpftAmt());
        
        return instance;
    }
    
    public static PlanReportEntityDO convert2DO(ReportEntity var) {
        if (var == null) {
            return null;
        }
        
        PlanReportEntityDO instance = new PlanReportEntityDO();
        
        instance.setAstNo(var.getAstNo());
        instance.setPlanNo(var.getPlanNo());
        instance.setUserId(var.getUserId());
        instance.setStatVal(var.getReportItem().getAstStatVal());
        instance.setStatType(var.getReportItem().getAstStatType().getCode());
        instance.setEventCode(var.getEventCode().getCode());
        int sign = 1;
        if (var.getEventCode().getAssetFlow() == AstFlowTypeEnum.FLOW_OUT) {
            sign = -1;
        }
        instance.setSign(sign);
        instance.setAstChg(var.getAstChg());
        instance.setAccountDate(var.getAccountDate());
        instance.setFeeAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setIntrAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setPrinAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN));
        instance.setOvdFeeAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.OVD_FEE));
        instance.setOvdIntrAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.OVD_INTR));
        instance.setOvdPrinAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.OVD_PRIN));
        instance.setFeePenalAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
        instance.setIntrPenalAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setPrinPenalAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        // 其他费用
        instance.setOtherFeeAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.OTHER_FEE));
        instance.setOtherFeePenalAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.OTHER_FEE_PNY));
        
        instance.setCost(var.getCost());
        instance.setOrgDistpftAmt(var.getOrgDistPftAmt());
        instance.setRptDim(var.getReportDimType().getCode());
        
        return instance;
    }
}