/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */
package com.abssqr.plat.common.model.convertor.web;

import com.abssqr.plat.common.facade.enums.AstPackStatusEnum;
import com.abssqr.plat.common.facade.enums.AstPackTypeEnum;
import com.abssqr.plat.common.facade.enums.AstScRuleLeftCondEnum;
import com.abssqr.plat.common.facade.enums.AstScRuleRightCondEnum;
import com.abssqr.plat.common.facade.enums.AstScreenRelTypeEnum;
import com.abssqr.plat.common.facade.enums.AstScreenRuleTypeEnum;
import com.abssqr.plat.common.facade.enums.AstTypeEnum;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.facade.param.ast.AstPackOprParam;
import com.abssqr.plat.common.facade.result.pack.AstPackVO;
import com.abssqr.plat.common.facade.result.pack.AstScreenRuleVO;
import com.abssqr.plat.common.model.domain.ast.AstPack;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.convertor.web: AstPackWebConvert.java, v 0.1 2020-04-21 11:27 bangis.wangdf
 * Exp $
 */
public class AstPackWebConvert {
    public static AstPack param2Model(AstPackOprParam param) {
        AstPack result = new AstPack();

        result.setPackNo(param.getPackNo());
        result.setPackName(param.getPackName());
        result.setAvalAmt(param.getAvalAmtMoney());
        result.setAstPoolNo(param.getAstPoolNo());
        result.setPackType(AstPackTypeEnum.NORMAL);
        result.setStatus(AstPackStatusEnum.INIT);
        result.setAstType(AstTypeEnum.getByCode(param.getAstType()));
        if (CollectionUtils.isNotEmpty(param.getScreenRuleList())) {
            param.getScreenRuleList().forEach(item -> {
                if (StringUtils.isNotEmpty(item.getLeftValue()) || StringUtils.isNotEmpty(item.getRightValue())) {
                    AstScreenRule rule = new AstScreenRule();
                    result.getScreenRuleList().add(rule);
                    //relNo 关联编号
                    rule.setRelNo(param.getPackNo());
                    //relType 关联类型 1.资产打包 PACK,2数据仓库 DW
                    rule.setRelType(AstScreenRelTypeEnum.PACK);
                    //ruleType 规则类型
                    rule.setRuleType(AstScreenRuleTypeEnum.getByCode(item.getRuleType()));
                    //leftValue 左值
                    rule.setLeftValue(item.getLeftValue());
                    //rightValue 右值
                    rule.setRightValue(item.getRightValue());
                    //leftCondition 左条件: 1.大于 GT 2.大于等于 GE
                    rule.setLeftCondition(AstScRuleLeftCondEnum.getByCode(item.getLeftCondition()));
                    //rightCondition 右条件: 1.小于 LT 2.小于等于 LE 3.等于 EQ 4.包含 IN
                    rule.setRightCondition(AstScRuleRightCondEnum.getByCode(item.getRightCondition()));
                }
            });
        }
        return result;
    }

    public static AstPackVO model2VO(AstPack param) {
        AstPackVO result = new AstPackVO();
        if (param != null) {
            //avalAmt 规模
            result.setAvalAmt(param.getAvalAmt());
            //type 资产包类型: 1. 正常 NORMAL 2.默认(虚拟) DEFAULT
            result.setPackType(param.getPackType());
            //packNo 资产包编号
            result.setPackNo(param.getPackNo());
            //planNo 产品编号
            result.setPlanNo(param.getPlanNo());
            //status 状态 1.初始化 INIT 2.筛选中 EXEC 3.筛选完成 SCREEN_FIS  4.已封包 PACK 5.已解包 UNPACK 6.已拆包 BREAK 7.已发行 PUBLISH
            result.setStatus(param.getStatus() == AstPackStatusEnum.INIT ? AstPackStatusEnum.EXEC : param.getStatus());
            //packName 资产包名称
            result.setPackName(param.getPackName());
            //astPoolNo 资产池编号
            result.setAstPoolNo(param.getAstPoolNo());
            //packDate 封包日
            result.setPackDate(param.getPackDate());
            //packSubmitDate 封包提交日
            result.setPackSubmitDate(param.getPackSubmitDate());
            if (CollectionUtils.isNotEmpty(param.getScreenRuleList())) {
                param.getScreenRuleList().forEach(astScreenRule -> {
                    result.getScreenRules().add(param2RVO(astScreenRule));
                });
            }
        }
        return result;
    }

    public static AstScreenRuleVO param2RVO(AstScreenRule param) {
        AstScreenRuleVO result = new AstScreenRuleVO();
        if (param != null) {
            result.setRuleType(param.getRuleType().getCode());
            result.setLeftCondition(param.getLeftCondition() == null ? null : param.getLeftCondition().getCode());
            result.setRightCondition(param.getRightCondition() == null ? null : param.getRightCondition().getCode());
            result.setLeftValue(param.getLeftValue());
            result.setRightValue(param.getRightValue());
        }
        return result;
    }
}
