package com.abssqr.plat.common.model.convertor.web;

import com.abssqr.plat.common.facade.enums.*;
import com.abssqr.plat.common.facade.model.plan.FeeStructure;
import com.abssqr.plat.common.facade.model.plan.InvestStructure;
import com.abssqr.plat.common.facade.model.plan.Plan;
import com.abssqr.plat.common.facade.param.plan.PlanEditOprParam;
import com.abssqr.plat.common.facade.param.plan.sub.FeeStructureParam;
import com.abssqr.plat.common.facade.param.plan.sub.InvestStructureParam;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import org.apache.commons.collections.CollectionUtils;

import java.math.BigDecimal;

public class PlanWebConvert {
    public static Plan convertPlanParam(PlanEditOprParam planEditOprParam){
        Plan plan=new Plan();
        //分配周期
        plan.setAllocPeriod(planEditOprParam.getAllocPeriod());
        plan.setAllocPeriodMethod(PeriodTypeEnum.getByCode(planEditOprParam.getAllocPeriodMethod()));
        plan.setBeginDate(planEditOprParam.getBeginDate());
        plan.setCalcIntrType(CalcIntrTypeEnum.getByCode(planEditOprParam.getCalcIntrType()));
        plan.setEndDate(planEditOprParam.getEndDate());
        plan.setFirstAllocDate(planEditOprParam.getFirstAllocDate());
        plan.setFirstCalcDate(planEditOprParam.getFirstCalcDate());
        plan.setFirstPayDate(planEditOprParam.getFirstPayDate());
        plan.setFirstReportDate(planEditOprParam.getFirstReportDate());
        plan.setFirstSettleDate(planEditOprParam.getFirstSettleDate());
        plan.setPlanFullName(planEditOprParam.getPlanFullName());
        plan.setTotalAmt(planEditOprParam.getTotalAmt());
        plan.setPlanName(planEditOprParam.getPlanName());
        plan.setPlanNo(planEditOprParam.getPlanNo());
        plan.setTradePlace(planEditOprParam.getTradePlace());

        if(CollectionUtils.isNotEmpty(planEditOprParam.getInvestStructureParam())) {
            for (InvestStructureParam investStructure : planEditOprParam.getInvestStructureParam()) {
                plan.getInvestStructure().add(convertInvestStructureParam(plan.getPlanNo(),investStructure));
            }
        }
        if(CollectionUtils.isNotEmpty(planEditOprParam.getFeeStructureParam())){
            for(FeeStructureParam feeStructureParam:planEditOprParam.getFeeStructureParam()){
                plan.getFeeStructure().add(convertFeeStructure(plan.getPlanNo(),feeStructureParam));
            }
        }

        return plan;
    }

    private static InvestStructure convertInvestStructureParam(String planNo,InvestStructureParam investStructureParam){
        InvestStructure investStructure=new InvestStructure();
        investStructure.setEffectDate(investStructureParam.getEffectDate());
        investStructure.setEndDate(investStructureParam.getEndDate());
        investStructure.setExpectEndDate(investStructureParam.getExpectEndDate());
        investStructure.setGrade(investStructureParam.getGrade());
        investStructure.setIntrType(IntrTypeEnum.getByCode(investStructureParam.getIntrType()));
        investStructure.setPlanNo(planNo);
        investStructure.setAllocMethod(AllocMethodEnum.getByCode(investStructureParam.getAllocMethod()));

        investStructure.setOrder(investStructureParam.getOrder());
        investStructure.setLayerType(LayerTypeEnum.getByCode(investStructureParam.getLayerType()));
        investStructure.setLayerNo(SequenceUtil.genId(IdTypeEnum.INVESTSTRUCTURE));
        investStructure.setLayerName(investStructureParam.getLayerName());
        investStructure.setLayerIntr(investStructureParam.getLayerIntr());
        investStructure.setLayerAmt(investStructureParam.getLayerAmt());

        return investStructure;
    }


    private static FeeStructure convertFeeStructure(String planNo,FeeStructureParam feeStructureParam){
        FeeStructure feeStructure=new FeeStructure();
        feeStructure.setFeeCalcDefaultDate(feeStructureParam.getFeeCalcDefaultDate());
        feeStructure.setPlanNo(planNo);
        feeStructure.setPayLimitAmt(feeStructureParam.getPayLimitAmt());
        feeStructure.setOrder(feeStructureParam.getOrder());
        feeStructure.setFixedFeeAmt(feeStructureParam.getFixedFeeAmt());
        feeStructure.setFeeScaleType(FeeScaleTypeEnum.getByCode(feeStructureParam.getFeeScaleType()));
        feeStructure.setFeeRate(new BigDecimal(feeStructureParam.getFeeRate()));
        feeStructure.setFeeNo(SequenceUtil.genId(IdTypeEnum.FEE));
        feeStructure.setFeeCalcType(CalcTypeEnum.getByCode(feeStructureParam.getFeeCalcType()));

        feeStructure.setFeeName(feeStructureParam.getFeeName());
        return feeStructure;
    }


}
