/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */

package com.abssqr.plat.common.model.domain.ast;

import com.abssqr.plat.common.facade.enums.AstPackStatusEnum;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.facade.model.prod.LoanProduct;
import com.abssqr.plat.common.facade.model.rule.AstPriceRule;
import com.general.system.bpm.enums.ApproveStatusEnum;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 回购筛选包模型
 *
 * @author xiachenxiang
 * @version com.abssqr.plat.common.model.domain.ast: LoanAstBuyBackPack.java, v 0.1 2020-02-11 2:37 PM xiachenxiang Exp $
 */
public class LoanAstBuyBackPack extends Pack {
    /**
     * astPriceRuleList 定价规则
     */
    private List<AstPriceRule> astPriceRuleList = new ArrayList<>();
    /**
     * LoanProduct 产品
     */
    private LoanProduct        loanProduct      = new LoanProduct();
    /**
     * backOutCode 转出机构.
     */
    private String             backOutCode;
    /**
     * approveDate 审批完成日期.
     */
    private Date               approveDate;
    /**
     * approveStatus 审批状态.
     */
    private ApproveStatusEnum  approveStatus;
    /**
     * memo 备注.
     */
    private String             memo;
    /**
     * 资产包类型
     */
    private AstPackTradeTypeEnum packType = AstPackTradeTypeEnum.BUY_BACK_BIZ;
    
    public boolean canExec() {
        // 捞取对应回购筛选包校验 PACK_STATUS为SCEEN_FIS  和 APPROVE_STATUS为PASSED
        if (this.getPackStatus() == AstPackStatusEnum.SCEEN_FIS
                && this.getApproveStatus() == ApproveStatusEnum.PASSED){
            return true;
        }
        return false;
    }
    
    
    public List<AstPriceRule> getAstPriceRuleList() {
        return astPriceRuleList;
    }
    
    public void setAstPriceRuleList(List<AstPriceRule> astPriceRuleList) {
        this.astPriceRuleList = astPriceRuleList;
    }
    
    public LoanProduct getLoanProduct() {
        return loanProduct;
    }

    public void setLoanProduct(LoanProduct loanProduct) {
        this.loanProduct = loanProduct;
    }

    @Override
    public AstPackTradeTypeEnum getPackType() {
        return packType;
    }

    @Override
    public void setPackType(AstPackTradeTypeEnum packType) {
        this.packType = packType;
    }

    public String getBackOutCode() {
        return backOutCode;
    }

    public void setBackOutCode(String backOutCode) {
        this.backOutCode = backOutCode;
    }

    public Date getApproveDate() {
        return approveDate;
    }

    public void setApproveDate(Date approveDate) {
        this.approveDate = approveDate;
    }

    public ApproveStatusEnum getApproveStatus() {
        return approveStatus;
    }

    public void setApproveStatus(ApproveStatusEnum approveStatus) {
        this.approveStatus = approveStatus;
    }

    public String getMemo() {
        return memo;
    }

    public void setMemo(String memo) {
        this.memo = memo;
    }
    
    /**
     * 定价规则分组
     * @return
     */
    public Map<String, AstPriceRule> getAstPriceRuleMap() {
        return astPriceRuleList.stream().collect(Collectors.toMap(AstPriceRule::getRiskCl, item -> item));
    }
    
    
}
