/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.domain.ast;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.util.MessageUtil;
import com.google.common.collect.Lists;

import java.math.BigDecimal;
import java.util.List;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.profit.entity: AstProfitMeasure.java, v 0.1 2019-11-17 下午16:39 zhenxuan.luo Exp $ 
 */
public class ProfitMeasure extends BaseEntity {
    
    public static List<DebtMoneyFieldEnum> PFT_AMT_TYPES = Lists.newArrayList();
    
    static {
        PFT_AMT_TYPES.add(DebtMoneyFieldEnum.INTR);
        PFT_AMT_TYPES.add(DebtMoneyFieldEnum.PRIN_ODUE_PNY);
        PFT_AMT_TYPES.add(DebtMoneyFieldEnum.INTR_ODUE_PNY);
        PFT_AMT_TYPES.add(DebtMoneyFieldEnum.FEE);
        PFT_AMT_TYPES.add(DebtMoneyFieldEnum.FEE_PNY);
        PFT_AMT_TYPES.add(DebtMoneyFieldEnum.INTR_ODUE_PNY);
        PFT_AMT_TYPES.add(DebtMoneyFieldEnum.OTHER_FEE);
    }
    
    protected AstProfitUnit intrEntry      = new AstProfitUnit(DebtMoneyFieldEnum.INTR);
    protected AstProfitUnit prinPenalEntry = new AstProfitUnit(DebtMoneyFieldEnum.PRIN_ODUE_PNY);
    protected AstProfitUnit intrPenalEntry = new AstProfitUnit(DebtMoneyFieldEnum.INTR_ODUE_PNY);
    protected AstProfitUnit feeEntry       = new AstProfitUnit(DebtMoneyFieldEnum.FEE);
    protected AstProfitUnit feePenalEntry  = new AstProfitUnit(DebtMoneyFieldEnum.FEE_PNY);
    protected AstProfitUnit otherFeeEntry  = new AstProfitUnit(DebtMoneyFieldEnum.OTHER_FEE);

    protected ProfitMeasure() {
    }
    
    public DebtMoney filterTransEntryAmt() {
        DebtMoney debtMoney = new DebtMoney();
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR, intrEntry.getTransEntryAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, intrPenalEntry.getTransEntryAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, prinPenalEntry.getTransEntryAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE, feeEntry.getTransEntryAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, feePenalEntry.getTransEntryAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE,otherFeeEntry.getTransEntryAmt());
        return debtMoney;
    }
    
    public BigDecimal getTransDecSum() {
        return intrEntry.getTransEntryDec().add(prinPenalEntry.getTransEntryDec())
                       .add(intrPenalEntry.getTransEntryDec())
                       .add(feeEntry.getTransEntryDec()).add(feePenalEntry.getTransEntryDec())
                        .add(otherFeeEntry.getTransEntryDec());
    }
    
    public DebtMoney filterEntryAmt() {
        DebtMoney debtMoney = new DebtMoney();
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR, intrEntry.getEntryAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, intrPenalEntry.getEntryAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, prinPenalEntry.getEntryAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE, feeEntry.getEntryAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, feePenalEntry.getEntryAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE,otherFeeEntry.getEntryAmt());
        return debtMoney;
    }
    
    public AstProfitUnit getUnit(DebtMoneyFieldEnum fieldType) {
        switch (fieldType) {
            case INTR:
                return intrEntry;
            case PRIN_ODUE_PNY:
                return prinPenalEntry;
            case INTR_ODUE_PNY:
                return intrPenalEntry;
            case FEE:
                return feeEntry;
            case FEE_PNY:
                return feePenalEntry;
            case OTHER_FEE:
                return otherFeeEntry;
            default:
                throw new AbssqrBizException(MessageUtil.formatMsg("不支持的息费计算类型[{0}]", fieldType),
                        CommonErrorCodeEnum.INVALID_INTERNAL_MODEL, false);
        }
    }
    
    public AstProfitUnit getIntrEntry() {
        return intrEntry;
    }
    
    public AstProfitUnit getPrinPenalEntry() {
        return prinPenalEntry;
    }
    
    public AstProfitUnit getIntrPenalEntry() {
        return intrPenalEntry;
    }
    
    public AstProfitUnit getFeeEntry() {
        return feeEntry;
    }
    
    public AstProfitUnit getFeePenalEntry() {
        return feePenalEntry;
    }

    public AstProfitUnit getOtherFeeEntry() {
        return otherFeeEntry;
    }
}