/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.model.domain.loanTrans;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.constants.LoanExtInfConstants;
import com.abssqr.plat.common.facade.enums.AstFlowTypeEnum;
import com.abssqr.plat.common.facade.enums.AstRiskClassEnum;
import com.abssqr.plat.common.facade.enums.CapitalAstMatchEnum;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanAstStatusEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanAst;
import com.abssqr.plat.common.model.domain.ast.LoanAstProfit;
import com.abssqr.plat.common.model.domain.ast.LoanBak;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 资产变更明细
 *
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.domain.loanTrans: LoanTransDetail.java, v 0.1 2017-12-12 下午8:20 guozhenhua Exp $
 */
public class LoanTransDetail implements ITransDtl {

    /**
     * 贷款资产
     */
    private Loan                        loan;
    /**
     * 贷款资产分期变更流水
     */
    private List<LoanInstalTransDetail> loanInstalTransDetails = Lists.newArrayList();
    /**
     * 资产流水
     */
    private LoanLog                     activeLog;
    /**
     * 判断是否当天初笔交易
     */
    private boolean                     isFirstTransOfDay      = false;
    /**
     *
     */
    private boolean                     isUpdate               = false;
    /**
     *
     */
    private boolean                     isInTrans              = true;

    private LoanTransDetail() {

    }

    /**
     * 构建一个贷款资产
     *
     * @param loan
     * @param transInfo
     * @return
     */
    public static LoanTransDetail createLoanAstTxnDtl(Loan loan, TransInfoTemplate transInfo) {

        if (loan.getActiveTransDtl() != null && loan.getActiveTransDtl().isInTrans) {
            throw new AbssqrBizException(MessageUtil.formatMsg("贷款[{0}]在交易中[{1}]", loan.getAstNo(),
                    loan.getActiveTransDtl().activeLog.getSeqNo()));
        }

        // 1. 构造资产交易明细
        LoanTransDetail loanTransDetail = new LoanTransDetail();
        loan.addLoanTransDtl(loanTransDetail);
        loan.setActiveTransDtl(loanTransDetail);

        // 2. 构造交易流水
        LoanLog loanLog = buildLoanLog(loan, transInfo);
        loanTransDetail.loan = loan;
        loanTransDetail.activeLog = loanLog;

        // 3. 处理初笔交易
        if (loan.getLastUpdateDate() == null || DateTimeUtil.isBeforeOnDate(loan.getLastUpdateDate(),
                transInfo.getAccountDate())) {
            loanTransDetail.isFirstTransOfDay = true;
            loan.setLastUpdateDate(transInfo.getAccountDate());
        }

        return loanTransDetail;
    }

    public static List<LoanBak> buildBaks(Loan loan, Date backDate) {
        Map<String, List<LoanAst>> loanAstMap = Maps.newHashMap();
        for (LoanInstalDomain instalDomain : loan.getLoanInstalList()) {
            for (LoanAst loanAst : instalDomain.getLoanAsts()) {
                String ownOrg = loanAst.getOwnOrg();
                if (!loanAstMap.containsKey(ownOrg)) {
                    loanAstMap.put(ownOrg, Lists.newArrayList());
                }
                loanAstMap.get(ownOrg).add(loanAst);
            }
        }

        Map<String, LoanBak> loanBakMap = Maps.newHashMap();
        List<LoanBak> ret = Lists.newArrayList();
        for (String ownOrg : loanAstMap.keySet()) {

            LoanBak loanBak = new LoanBak();
            loanBak.setRate(loan.getContract().getLoanProperty().getRate());
            loanBak.setRateType(loan.getContract().getLoanProperty().getRateType());
            loanBak.setRateDays(loan.getContract().getLoanProperty().getRateDays());

            // 资产逾期天数
            loanBak.setAstStatus(loan.getStatus());
            loanBak.setAstOvdDays(loan.getLoanRisk().getOvdDays());
            loanBak.setBakDate(backDate);

            ret.add(loanBak);
            loanBakMap.put(ownOrg, loanBak);

            Loan orgLoan = loan.cloneSimple();
            loanBak.setLoan(orgLoan);
            boolean isAllClear = true;
            boolean isOverdue = false;
            Date maxClearDate = null;

            // 重置逾期天数，开始日期，到期日期，在后面通过分期计算
            orgLoan.getLoanRisk().setOvdDays(0);
            orgLoan.getAstPeriod().setStartDate(null);
            orgLoan.getAstPeriod().setEndDate(null);
            orgLoan.getAstPeriod().setInDate(null);
            orgLoan.getAstPeriod().setClearDate(null);
            for (LoanAst loanAst : loanAstMap.get(ownOrg)) {
                LoanInstalDomain loanInstal = loanAst.getRefInstal();
                orgLoan.getAstOrg().cloneTo(loanInstal.getAstOrg());
                orgLoan.getAstOrg().getAstOwnership().setOwnOrg(loanAst.getOwnOrg());
                orgLoan.getAstOrg().getAstOwnership().setLenderOrgCode(loanAst.getLenderOrgCode());
                orgLoan.getAstOrg().getAstOwnership().setLastOwnOrg(loanAst.getLenderOrgCode());
                orgLoan.getAstOrg().getAstOwnership().setOwnStatus(loanAst.getOwnStatus());

                // 本金余额:
                // 本金状态以LoanAst为准，利息状态以利息实际所在的LoanInstal为准
                // 备份状态：如逾期天数，状态等以分期状态为准，所以可能存在：带逾期利息的正常LoanBak
                DebtMoney bal = new DebtMoney();
                DebtMoney paid = new DebtMoney();
                // 根据机构资产持有正常或结清状态下记录成本
                if (loanAst.getAstStatus() != LoanAstStatusEnum.OUT){
                    loanBak.getCost().addTo(loanAst.getCostAmt());
                }
                
                bal.setByFieldType(DebtMoneyFieldEnum.PRIN, loanAst.getPrinBal());
                paid.setByFieldType(DebtMoneyFieldEnum.PRIN, loanAst.getPaidPrinAmt());

                orgLoan.getBal().addAll(getAmtFromInstall(bal, loanInstal.getStatus()));
                orgLoan.getPaid().addAll(getAmtFromInstall(paid, loanInstal.getStatus()));

                if (orgLoan.getLoanRisk().getOvdDays() < loanInstal.getOvdDays()) {
                    orgLoan.getLoanRisk().setOvdDays(loanInstal.getOvdDays());
                }

                // 期限
                if (orgLoan.getAstPeriod().getEndDate() == null || DateTimeUtil.calcDayDiff(
                        orgLoan.getAstPeriod().getEndDate(), loanInstal.getAstPeriod().getEndDate()) < 0) {
                    orgLoan.getAstPeriod().setEndDate(loanInstal.getAstPeriod().getEndDate());
                }
                if (orgLoan.getAstPeriod().getStartDate() == null
                        || DateTimeUtil.calcDayDiff(orgLoan.getAstPeriod().getStartDate(),
                        loanInstal.getAstPeriod().getStartDate()) > 0) {
                    orgLoan.getAstPeriod().setStartDate(loanInstal.getAstPeriod().getStartDate());
                }
                if (orgLoan.getAstPeriod().getInDate() == null
                        || DateTimeUtil.calcDayDiff(orgLoan.getAstPeriod().getInDate(), loanAst.getInDate()) > 0) {
                    orgLoan.getAstPeriod().setInDate(loanAst.getInDate());
                }
                if (loanInstal.getAstPeriod().getClearDate() != null) {
                    // 即使LoanAst状态不为Normal（如out），也有可能会产生分润，不能当做结清
                    if (maxClearDate == null
                            || DateTimeUtil.calcDayDiff(maxClearDate, loanInstal.getAstPeriod().getClearDate()) < 0) {
                        maxClearDate = loanInstal.getAstPeriod().getClearDate();
                    }
                }

                // 状态
                if (loanInstal.getStatus() != LoanStatusEnum.CLEAR) {
                    isAllClear = false;
                }

                if (loanInstal.getStatus() == LoanStatusEnum.OVERDUE) {
                    isOverdue = true;
                }
            }

            if (loan.getStatus() == LoanStatusEnum.REVERSE) {
                orgLoan.setStatus(LoanStatusEnum.REVERSE);
            } else if (isAllClear) {
                orgLoan.setStatus(LoanStatusEnum.CLEAR);
                orgLoan.getAstPeriod().setClearDate(maxClearDate);
            } else if (isOverdue) {
                orgLoan.setStatus(LoanStatusEnum.OVERDUE);
            } else {
                orgLoan.setStatus(LoanStatusEnum.NORMAL);
            }
        }

        for (LoanInstalDomain loanInstal : loan.getLoanInstalList()) {
            for (LoanAstProfit astProfit : loanInstal.getSourcedProfits()) {
                DebtMoney bal = astProfit.getBal().cloneAll();
                DebtMoney paid = astProfit.getPaidAmt().cloneAll();

                LoanBak loanBak = loanBakMap.get(astProfit.getOwnOrg());
                loanBak.getLoan().getBal().addAll(getAmtFromInstall(bal, loanInstal.getStatus()));
                loanBak.getLoan().getPaid().addAll(getAmtFromInstall(paid, loanInstal.getStatus()));
            }
        }

        DebtMoney allBal = new DebtMoney();
        for (LoanBak loanBak : ret) {
            allBal.addAll(loanBak.getLoan().getBal());
        }

        VarChecker.checkEquals(allBal, loan.getBal(), "贷款[{0}]备份余额汇总[{1}]和贷款实际余额[{2}]不相等", loan.getAstNo(), allBal, loan.getBal());

        return ret;
    }

    private static LoanLog buildLoanLog(Loan loan, TransInfoTemplate transInfo) {

        LoanLog loanLog = new LoanLog();
        // 2.1. 资产信息
        loanLog.setAstNo(loan.getAstNo());
        loanLog.setUserId(loan.getUserId());
        loanLog.setOutAstNo(loan.getOutAstNo());
        loanLog.setLenderOrgCode(loan.getAstOrg().getAstOwnership().getLenderOrgCode());
        loanLog.setManagerOrgCode(loan.getAstOrg().getManagerOrgCode());
        loanLog.setSponserOrgCode(loan.getAstOrg().getSponsorOrgCode());
        loanLog.setOwnOrg(loan.getAstOrg().getAstOwnership().getOwnOrg());
        loanLog.setOwnPack(loan.getOwnPack());
        loanLog.setProdNo(loan.getProdNo());
        loanLog.setOutProdNo(loan.getOutProdNo());
        loanLog.setContractNo(loan.getContNo());

        // 2.2. 交易信息
        loanLog.setOutBizNo(transInfo.getOutBizNo());
        loanLog.setBizNo(transInfo.getBizNo());
        loanLog.setSeqNo(SequenceUtil.genSeqByLoan(loan, transInfo.getEventCode().getSceneCode()));
        loanLog.setTransTime(transInfo.getTransTime());
        loanLog.setAccountDate(transInfo.getAccountDate());
        loanLog.setEventCode(transInfo.getEventCode());
        loanLog.setExecTms(transInfo.getTimestamp());

        // 2.3. 交易前数据备份
        loanLog.setBfRiskCl(loan.getLoanRisk().getRiskClass());
        loanLog.setBfOvdDays(loan.getLoanRisk().getOvdDays());
        loanLog.setBfTermCnt(loan.getTermCnt());
        loanLog.setBfEndDate(loan.getAstPeriod().getEndDate());
        loanLog.setBfStatus(loan.getStatus());
        loanLog.setBfOwnStatus(loan.getAstOrg().getAstOwnership().getOwnStatus());
        loanLog.setBfBal(loan.getBal().cloneAll());
        loanLog.setBfWtfStatus(loan.getLoanRisk().getWtfStatus());
        loanLog.setBfCapitalNo(loan.getCapitalNo());
        return loanLog;
    }

    private static DebtMoney getAmtFromInstall(DebtMoney amt, LoanStatusEnum astStatus) {

        DebtMoney loanAmt = new DebtMoney();
        for (DebtMoneyFieldEnum debtMoneyField : DebtMoneyFieldEnum.values()) {
            DebtMoneyFieldEnum loanDebtMoneyField = debtMoneyField;
            if (astStatus == LoanStatusEnum.OVERDUE) {
                switch (debtMoneyField) {
                    case PRIN:
                        loanDebtMoneyField = DebtMoneyFieldEnum.OVD_PRIN;
                        break;
                    case INTR:
                        loanDebtMoneyField = DebtMoneyFieldEnum.OVD_INTR;
                        break;
                    case FEE:
                        loanDebtMoneyField = DebtMoneyFieldEnum.OVD_FEE;
                        break;
                    default:
                        break;
                }
            }
            loanAmt.addByFieldType(loanDebtMoneyField, amt.getByFieldType(debtMoneyField));
        }

        return loanAmt;
    }

    public LoanInstalTransDetail addLoanInstalTransDtl(LoanInstalDomain loanInstal) {
        return addLoanInstalTransDtl(loanInstal, null);
    }

    public LoanInstalTransDetail addLoanInstalTransDtl(LoanInstalDomain loanInstal, EventCodeEnum eventCode) {
        checkIntrans();
        LoanInstalTransDetail loanInstalTransDetail = LoanInstalTransDetail.createLoanInstalTxnDtl(loanInstal, this);
        loanInstalTransDetails.add(loanInstalTransDetail);

        if (null != eventCode) {
            loanInstalTransDetail.getActiveLog().setEventCode(eventCode);
        }
        return loanInstalTransDetail;
    }

    @Override
    public void processBeforeUpdate() {
        checkIntrans();
        // 将交易后信息填入交易流水
        activeLog.setAfBal(loan.getBal().cloneAll());

        activeLog.setAfTermCnt(loan.getTermCnt());
        activeLog.setAfOvdDays(loan.getLoanRisk().getOvdDays());
        activeLog.setAfEndDate(loan.getAstPeriod().getEndDate());
        activeLog.setAfRiskCl(loan.getLoanRisk().getRiskClass());
        activeLog.setAfStatus(loan.getStatus());
        activeLog.setAfOwnStatus(loan.getAstOrg().getAstOwnership().getOwnStatus());
        activeLog.setAfWtfStatus(loan.getLoanRisk().getWtfStatus());
        activeLog.setAfCapitalNo(loan.getCapitalNo());
        for (LoanInstalTransDetail loanInstalTransDetail : this.getLoanInstalTransDetails()) {
            loanInstalTransDetail.processBeforeUpdate();
        }

        isInTrans = false;
    }

    /**
     * 设置逾期天数
     *
     * @param ovdDays
     */
    public void setOvdDays(long ovdDays) {
        checkIntrans();
        if (loan.getLoanRisk().getOvdDays() == ovdDays) {
            return;
        }
        isUpdate = true;
        loan.getLoanRisk().setOvdDays((int) ovdDays);
    }

    /**
     * 设置逾期天数
     *
     * @param curTerm
     */
    public void setCurTerm(int curTerm) {
        checkIntrans();
        if (loan.getCurTerm() == curTerm) {
            return;
        }
        isUpdate = true;
        loan.setCurTerm(curTerm);
    }

    /**
     * 设置五级分类
     *
     * @param astRiskClass
     */
    public void setAstCl(AstRiskClassEnum astRiskClass) {
        if (loan.getLoanRisk().getRiskClass() == astRiskClass) {
            return;
        }
        checkIntrans();
        isUpdate = true;
        loan.getLoanRisk().setRiskClass(astRiskClass);
    }

    /**
     * 结清日期
     *
     * @param clearDate
     */
    public void setClearDate(Date clearDate) {
        if (this.loan.getAstPeriod().getClearDate() != null) {
            return;
        }
        checkIntrans();
        this.isUpdate = true;
        if (this.loan.getAstPeriod() != null) {
            this.loan.getAstPeriod().setClearDate(clearDate);
        }
    }

    /**
     * 设置资产状态
     *
     * @param status
     */
    public void setStatus(LoanStatusEnum status) {
        if (loan.getStatus() == status) {
            return;
        }
        checkIntrans();
        this.isUpdate = true;
        this.loan.setStatus(status);
    }

    /**
     * 设置核销状态
     */
    public void writeOff() {
        checkIntrans();
        if (loan.getLoanRisk().getWtfStatus() == YNEnum.Y) {
            return;
        }
        this.isUpdate = true;
        this.loan.getLoanRisk().setWtfStatus(YNEnum.Y);
    }

    public LoanLog getActiveLog() {
        return activeLog;
    }

    public boolean isUpdate() {
        return isUpdate;
    }

    public Loan getLoan() {
        return loan;
    }

    public List<LoanInstalTransDetail> getLoanInstalTransDetails() {
        return loanInstalTransDetails;
    }

    /**
     * 设置清算金额
     *
     * @param clrgAmt
     */
    protected void changeClrgAmt(Money clrgAmt) {
        this.activeLog.getClrgAmt().addTo(clrgAmt);
    }

    /**
     * 处理发生额
     *
     * @param transAmt
     */
    protected void processTransAmtFromInstal(DebtMoney transAmt, LoanStatusEnum statusEnum, EventCodeEnum eventCode) {

        if (this.activeLog.getEventCode().getSceneCode() == TransCodeEnum.REPO) {
            this.loan.getPaid().addAll(transAmt);
        }

        DebtMoney loanAmt = getAmtFromInstall(transAmt, statusEnum);

        changeBalByAmt(loanAmt, eventCode.getAssetFlow());
    }

    /**
     * @param amt
     * @param isUpdate
     */
    protected void addTransAmtOnlyFromInstal(DebtMoney amt, LoanStatusEnum astStatus, boolean isUpdate) {
        DebtMoney loanAmt = getAmtFromInstall(amt, astStatus);
        this.activeLog.getAmt().addAll(loanAmt);
        this.isUpdate = this.isUpdate || isUpdate;
    }

    /**
     * 负数表示逾期金额
     *
     * @param amt
     */
    protected void changeStatusAmtForTurnOdue(DebtMoney amt) {

        DebtMoney loanAmt = new DebtMoney();
        loanAmt.setByFieldType(DebtMoneyFieldEnum.PRIN, amt.getByFieldType(DebtMoneyFieldEnum.PRIN).multiply(-1));
        loanAmt.setByFieldType(DebtMoneyFieldEnum.OVD_PRIN, amt.getByFieldType(DebtMoneyFieldEnum.PRIN));
        loanAmt.setByFieldType(DebtMoneyFieldEnum.INTR, amt.getByFieldType(DebtMoneyFieldEnum.INTR).multiply(-1));
        loanAmt.setByFieldType(DebtMoneyFieldEnum.OVD_INTR, amt.getByFieldType(DebtMoneyFieldEnum.INTR));
        loanAmt.setByFieldType(DebtMoneyFieldEnum.FEE, amt.getByFieldType(DebtMoneyFieldEnum.FEE).multiply(-1));
        loanAmt.setByFieldType(DebtMoneyFieldEnum.OVD_FEE, amt.getByFieldType(DebtMoneyFieldEnum.FEE));

        changeBalByAmt(loanAmt, AstFlowTypeEnum.FLOW_IN);

    }

    private void changeBalByAmt(DebtMoney amt, AstFlowTypeEnum astFlowTypeEnum) {
        isUpdate = true;
        DebtMoney curBal = this.loan.getBal();
        DebtMoney newBal = new DebtMoney();

        this.activeLog.getAmt().addAll(amt);
        // 1. 根据事件码，设定符号
        int sign = astFlowTypeEnum.getSign();

        // 2. 计算交易后余额
        for (DebtMoneyFieldEnum debtMoneyField : DebtMoneyFieldEnum.values()) {
            Money curBalField = curBal.getByFieldType(debtMoneyField);
            Money transAmtField = amt.getByFieldType(debtMoneyField);
            Money newBalField = curBalField.add(transAmtField.multiply(sign));
            newBal.setByFieldType(debtMoneyField, newBalField);
        }

        // 3. 回填余额
        this.loan.setBal(newBal);
    }

    public void setUpdate(boolean isUpdate) {
        this.isUpdate = isUpdate;
    }

    private void checkIntrans() {
        VarChecker.checkArgument(isInTrans, "贷款[{0}]的交易[{1}]已经结束处理，不允许变更", activeLog.getAstNo(), activeLog.getSeqNo());
    }

    /**
     * 设置资金资产匹配信息
     *
     * @param capitalNo
     * @param layerName
     */
    public void setCapitalInfo(String capitalNo, String layerName) {
        checkIntrans();
        this.isUpdate = true;
        if (StringUtils.equals(capitalNo, CapitalAstMatchEnum.DEFAULT.getCode())) {
            loan.getExtInf().put(LoanExtInfConstants.BF_CAPITAL_NO, this.loan.getCapitalNo());
        }
        this.loan.setCapitalNo(capitalNo);
        this.loan.setLayerName(layerName);
    }
}
