/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2020 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.domain.rule;

import com.abssqr.plat.common.facade.model.rule.TradePriceRule;
import com.abssqr.plat.common.model.domain.ast.LoanAstProfit;
import com.abssqr.plat.common.model.utils.TextUtil;
import com.abssqr.plat.common.util.utils.BeanCopierUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Maps;

import java.util.Map;

/**
 * 资产定价结果
 * @author hanfei
 * @version $Id: LoanAstPriceResult.java, v 0.1 2020-02-08 10:34 PM hanfei Exp $ 
 */
public class LoanAstPriceResult extends LoanAstBalAndPrice {
    
    /**
     * 转入机构
     */
    private String              transIn;
    /**
     * 转出机构
     */
    private String              transOut;
    /**
     * 定价规则
     */
    private TradePriceRule tradePriceRule;
    /**
     * 收益余额余额与定价金额
     */
    private Map<String, LoanAstProfitBalAndPrice> profitBalAndPriceMap = Maps.newHashMap();
    
    /**
     * 获取对应收益的余额与定价金额
     * @param astProfit
     * @return
     */
    public LoanAstProfitBalAndPrice getProfitBalAndPrice(LoanAstProfit astProfit){
        LoanAstProfitBalAndPrice balAndPrice = profitBalAndPriceMap.get(astProfit.getAstProfitNo());
        VarChecker.checkNotNull(balAndPrice,
                "未找到定价数据机构[{0}]资产[{1}]对应loanAst[{2}]termNo{3}-srcTermNo{4}",
                astProfit.getOwnOrg(), astProfit.getAstNo(), astProfit.getLoanAstNo(),
                astProfit.getTermNo(), astProfit.getSrcTermNo());
        VarChecker.checkNotNull(balAndPrice,
                "定价余额与当前余额不一致机构[{0}]资产[{1}]对应loanAst[{2}]termNo{3}-srcTermNo{4},[loan:{5}-price:{6}]",
                astProfit.getOwnOrg(), astProfit.getAstNo(), astProfit.getLoanAstNo(),
                astProfit.getTermNo(), astProfit.getSrcTermNo(), astProfit.getBal().getSum(), balAndPrice.getBal().getSum());
//        // 获取到空的定价创建默认
//        if (balAndPrice == null){
//            balAndPrice = new LoanAstProfitBalAndPrice(astProfit.getBal(), astProfit.getBal());
//        }
        return balAndPrice;
    }
    
    public String getExtText(){
        Map<String, Object> extInfo = Maps.newHashMap();
        extInfo.put("bal", BeanCopierUtil.beanToMap(this.getBal()));
        extInfo.put("price", BeanCopierUtil.beanToMap(this.getBal()));
        extInfo.put("rule", BeanCopierUtil.beanToMap(this.getTradePriceRule()));
        extInfo.put("transIn", transIn);
        extInfo.put("transOut", transOut);
        return TextUtil.convertMap2Txt(extInfo);
    }
    
    public TradePriceRule getTradePriceRule() {
        return tradePriceRule;
    }
    
    public void setTradePriceRule(TradePriceRule tradePriceRule) {
        this.tradePriceRule = tradePriceRule;
    }
    
    public String getTransIn() {
        return transIn;
    }
    
    public void setTransIn(String transIn) {
        this.transIn = transIn;
    }
    
    public String getTransOut() {
        return transOut;
    }
    
    public void setTransOut(String transOut) {
        this.transOut = transOut;
    }
    
    public Map<String, LoanAstProfitBalAndPrice> getProfitBalAndPriceMap() {
        return profitBalAndPriceMap;
    }
    
    public void setProfitBalAndPriceMap(
            Map<String, LoanAstProfitBalAndPrice> profitBalAndPriceMap) {
        this.profitBalAndPriceMap = profitBalAndPriceMap;
    }
    
}
