/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.task;

import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtrLog;
import com.abssqr.plat.common.util.base.Day;

import java.util.Date;
import java.util.List;

/**
 * 资产同步
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.repo.task.ast.sync: AstSyncTaskCtrRepo.java, v 0.1 2019-06-14 10:41 bangis
 * .wangdf Exp $
 */
public interface AstSyncTaskCtrRepo {
    /**
     * 根据机构获取控制
     *
     * @param
     * @return
     */
    List<AstSyncTaskCtr> getAstSyncTaskCtrs();

    /**
     * 根据机构获取控制
     *
     * @param orgCode
     * @return
     */
    AstSyncTaskCtr getAstSyncTaskCtr(String orgCode);
    
    /**
     * 根据机构获取控制
     *
     * @param orgCode
     * @return
     */
    AstSyncTaskCtr lockAstSyncTaskCtr(String orgCode);
    
    /**
     * 插入同步日志
     * @param execDate  执行日- 转让会计日
     * @param orgDate   机构业务会计日
     * @param astSyncTaskCtr 同步任务控制
     * @param taskType  任务类型
     * @param hostIp    IP
     * @return
     */
    String insertSyncLog(Date execDate, Day orgDate, AstSyncTaskCtr astSyncTaskCtr, SyncTaskTypeEnum taskType, String hostIp);
    
    String insertSyncLog(Date execDate, Day orgDate, AstSyncTaskCtr astSyncTaskCtr, SyncTaskTypeEnum taskType, TaskStatusEnum status, String hostIp);

    /**
     * 更新日志记录
     *
     * @param hostIp
     * @param taskLogCode
     * @param status
     * @param oldStatus
     */
    void updateSyncLogStatus(String hostIp, String taskLogCode, SyncTaskTypeEnum taskType, TaskStatusEnum status,
                             TaskStatusEnum oldStatus);
    
    /**
     * 根据机构日进行判断 任务是否可以运行
     * @param orgCode
     * @param taskCode
     * @param taskType
     * @param orgDate
     * @return
     */
    boolean checkCanRun(String orgCode, String taskCode, SyncTaskTypeEnum taskType, Date orgDate);

    
    /**
     * 根据机构获取控制
     *
     * @param execDate
     * @param taskType
     * @param orgCode
     * @return
     */
    AstSyncTaskCtrLog getTaskLogByExecDate(Date execDate, SyncTaskTypeEnum taskType, String orgCode);


    /**
     * 根据机构获取控制
     *
     * @param taskType
     * @param orgCode
     * @return
     */
    AstSyncTaskCtrLog getLastTaskLogByOrgDate(Date execDate,SyncTaskTypeEnum taskType, String orgCode);
    
    /**
     * 根据机构号、机构会计日获取对应日期的任务类型日志
     * @param orgDate
     * @param taskType
     * @param orgCode
     * @return
     */
    AstSyncTaskCtrLog getTaskLogByOrgDate(Date orgDate, SyncTaskTypeEnum taskType, String orgCode);
    
    /**
     * 获取已完成的任务,当前任务已完成，下一任务未开始
     *
     * @param acctDate
     * @param currTaskType
     * @return
     */
    List<String> getLastStatusFinishedOrgCodes(Date acctDate, SyncTaskTypeEnum currTaskType);

    /**
     * 审批通过后往同步中增加需要同步的放款计划
     * @param orgCode
     * @param entityCode
     */
    void addSyncSPV(String orgCode, String entityCode);
//    /**
//     * 发布前删除
//     * 获取指定类型的最大完成日期
//     * @param taskType
//     * @return
//     */
//    @Deprecated
//    Date getTaskLogMaxExecDate(SyncTaskTypeEnum taskType);

    /**
     * 获取指定类型的最大完成日期
     * @param taskType
     * @param managerOrgCode 管理机构
     * @return
     */
    Date getTaskLogMaxOrgDate(SyncTaskTypeEnum taskType, String  managerOrgCode);
}
