/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.task;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.CommonTaskDO;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * 通用任务
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.repo.task: CommonTaskRepository.java, v 0.1 2018-03-23 下午5:06 guozhenhua Exp $ 
 */
public interface CommonTaskRepository {

    /**
     * 创建通用任务
     * @param commonTask
     */
    void create(CommonTask commonTask);

    /**
     * 锁任务
     * @param id
     * @return
     */
    CommonTask lock(Long id);
    /**
     * desc:查询待执行的任务.<br/>
     *
     * @param curTime curTime
     * @return List<SyncAstBillDO>
     */
    List<CommonTask> selectInitTask(List<CommonTaskTypeEnum> taskTypes,Date curTime, int limit);
    
    /**
     *
     * @param taskType
     * @return
     */
    List<CommonTask> selectUnifinishTaskByType(CommonTaskTypeEnum taskType);

    /**
     * @param id
     * @param newTaskStatus
     * @param oldTaskStatus
     * @param memo
     * @return
     */
    int updateTaskStatus(Long id, TaskStatusEnum newTaskStatus, TaskStatusEnum oldTaskStatus, String memo);

    /**
     * desc:更新任务为执行中.<br/>
     *
     * @param lastExecTime lastExecTime
     * @param lastHost     lastHost
     * @param tasks        tasks
     * @return int
     */
    int updateTaskExec(Date lastExecTime, String lastHost, List<CommonTask> tasks);

    /**
     * desc:重置状态，更新下次执行时间.<br/>
     *
     * @param nextExecTime nextExecTime
     * @param id           id
     * @param memo
     * @return int
     */
    int updateTaskExep(Date nextExecTime, Long id, String memo);

    /**
     * 根据ip地址恢复悬挂任务
     *
     * @param ip
     * @return
     */
    int updateHangTasksByIp(String ip);

    List<CommonTask> queryInitAndExecUnMatchOrMatchTaskByContext(String planNo);

    List<CommonTask> queryExecUnMatchOrMatchTaskByContext(String planNo);

    List<CommonTask> queryInitUnMatchOrMatchTaskByContext(String planNo);

    List<CommonTask> queryIgnoreUnMatchOrMatchTaskByContext(String planNo);

    List<CommonTask> queryByBizNo(String bizNo);

    int updateIgnoreTaskStatus2Init(Long id);

    /**
     * 根据任务类型和任务状态查询
     * @param typeList
     * @param statusList
     * @return
     */
    List<CommonTask> queryByTypesAndStatus(List<String> typeList, List<String> statusList);

    /**
     * 根据任务类型、任务状态和执行时间范围计数
     * @param second
     * @param typeList
     * @param statusList
     * @return
     */
    Long countByTypesAndStatusAndRange(Long second,List<String> typeList,List<String> statusList);

    /**
     * 查询未完成的执行次数大于1的任务数量
     * @param typeList
     * @return
     */
    Long countUnFinishAndMultipleExecByTypes(List<String> typeList);
}
