/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.common.model.seq;

import com.abssqr.plat.common.dal.mysql.auto.dao.AbsSequenceDAO;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.general.system.common.util.SystemDateUtil;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Component;

import java.util.UUID;

/**
 * @author zhenxuan.luo
 * @version $Id: SequenceUtil.java, v 0.1 2017-06-22 下午15:42 zhenxuan.luo Exp $
 */
@Component
public class SequenceUtil implements ApplicationContextAware {
    
    private static ApplicationContext applicationContext = null;
    
    private static SequenceGenerator sequenceGenerator = null;
    private static AbsSequenceDAO    absSequenceDAO    = null;
    private static String            GENERATOR_NAME    = "sequenceGenerator";
    private static String            SEQUENCE_NAME     = "absSequenceDAO";
    
    public static String genSeqByLoan(Loan loan, TransCodeEnum transCode) {
        Long transCnt = loan.nextTransCnt();
        return sequenceGenerator.genLoanSeqNo(loan.getAstNo(), transCnt, transCode.getCode());
    }
    
    public static String genLoanId(Loan loan) {
        Long transCnt = loan.nextTransCnt();
        return sequenceGenerator.genLoanId(loan.getAstNo(), transCnt);
    }
    
    /**
     * 生成五矿流水号，要求一ZC开头，32位长度
     *
     * @return
     */
    public static String genWkSeqNo() {
        WkSeqStructure wkSeqStructure = new WkSeqStructure(SystemDateUtil.getSystemDate(), "ZCABSSQR",
                absSequenceDAO.getSeqText());
        return wkSeqStructure.toString();
    }
    
    /**
     * 生成五矿流水号，要求一ZC开头，32位长度
     *
     * @return
     */
    public static String genWkSeqNo(String seq) {
        WkSeqStructure wkSeqStructure = new WkSeqStructure(SystemDateUtil.getSystemDate(), seq,
                absSequenceDAO.getSeqText());
        return wkSeqStructure.toString();
    }
    
    /**
     * @return
     */
    public static String genFakeSeqNo() {
        return UUID.randomUUID().toString().toUpperCase();
    }
    
    public static String genSeqNo(TransCodeEnum transCode) {
        return sequenceGenerator.genSeqNo(IdTypeEnum.SEQ.getCode(), absSequenceDAO.getSeqText(), transCode.getCode());
    }
    
    public static String genSeqNo(String transCode) {
        return sequenceGenerator.genSeqNo(IdTypeEnum.SEQ.getCode(), absSequenceDAO.getSeqText(), transCode);
    }
    
    /**
     * 生成ID
     *
     * @param idName
     * @return
     */
    public static String genId(IdTypeEnum idName) {
        return sequenceGenerator.genId(idName.getCode(), absSequenceDAO.getSeqText());
    }
    
    @Override
    public void setApplicationContext(ApplicationContext context) throws BeansException {
        
        if (applicationContext == null) {
            SequenceUtil.applicationContext = context;
        }
        
        sequenceGenerator = (SequenceGenerator) context.getBean(GENERATOR_NAME);
        absSequenceDAO = (AbsSequenceDAO) context.getBean(SEQUENCE_NAME);
    }
    
}