/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.common.util.thread;

import com.general.system.common.util.SystemDateUtil;
import org.apache.commons.lang3.StringUtils;

import java.util.Date;

/**
 *
 * @author zhenxuan.luo
 * @version com.loan.common.model.domain.transList: BizContextHolder.java, v 0.1 2018-12-14 AM04:31 zhenxuan.luo Exp $
 */
public class BizContextHolder {
    
    /** 线程上下文 */
    private static final ThreadLocal<BizContext> HOLDER = new CleanableThreadLocal<BizContext>() {
        @Override
        protected BizContext initialValue() {
            return new BizContext();
        }
    };
    
    private BizContextHolder() {
    }
    
    public static String getBizNo() {
        return HOLDER.get().bizNo;
    }
    
    public static Date getTransTime() {
        return HOLDER.get().transTime;
    }
    
    public static String getEventCode() {
        return HOLDER.get().eventCode;
    }
    
    public static String getOperator() {
        return HOLDER.get().operator;
    }
    
    public static void set(String bizNo, String eventCode) {
        BizContext bizContext = new BizContext();
        bizContext.bizNo = bizNo;
        bizContext.eventCode = eventCode;
        bizContext.transTime = SystemDateUtil.getSystemDate();
        String operator = "" ;//IdentityUtil.getUser().getAccount();
        if (StringUtils.isBlank(operator)) {
            operator = "system";
        }
        bizContext.operator = operator;
        HOLDER.set(bizContext);
    }
    
    /**
     * 清理
     */
    public void clear() {
        HOLDER.remove();
    }
    
    /**
     * 线程上下文数据定义
     */
    private static class BizContext {
        
        private String bizNo;
        
        private Date transTime;
        
        private String eventCode;
        
        private String operator;
        
        /**
         * 默认构造函数，设置业务的操作时间
         */
        public BizContext() {
            this.transTime = SystemDateUtil.getSystemDate();
        }
    }
    
}